/*===========================================================================

ObjectWeb Browser Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

/** The Browser API's imports */
import org.objectweb.util.browser.core.common.DefaultTreePanel;
import org.objectweb.util.browser.core.common.DefaultViewPanel;
import org.objectweb.util.browser.core.common.DefaultStatusBar;
import org.objectweb.util.browser.core.common.DynamicTree;

/** The Fractal API's import */
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.NameController;

/** The java API's imports */
import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;
import java.awt.BorderLayout;
import java.awt.Dimension;

/*import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JMenuBar;
import javax.swing.JToolBar;
import javax.swing.ImageIcon;*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.URL;

/**
 * This Fractal component contains the browser
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BrowserImpl 
    implements BindingController, BrowserAttributes {
    
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The string which represents the fcAppl client */
    protected final String FC_APPL = "fcAppl";
    
    /** The associated tree */
    protected DynamicTree tree_;
   
    /** The array of the config files */
    protected List configFiles_ = new Vector();
    
    /** The  list of Component */
    protected Map elements_ = new HashMap();
    
    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    /**
     * Return the name of the node
     */
    protected String getName(Component component) {
        if(component!=null){
            NameController nameController = null;
            try{
                nameController = (NameController)component.getFcInterface("name-controller");
            }catch(NoSuchInterfaceException e){
                return "Root Component";        
            }
            if(nameController!=null)
                return nameController.getFcName();        
        }
        return "";
    }

    /** Returns an ImageIcon, or null if the path is invalid. */
    protected static ImageIcon createIcon(String imageName) {
        if (imageName != null) {
            URL urlFile = null;
            urlFile = Thread.currentThread().getContextClassLoader().getResource(imageName);            
            if(urlFile==null){            
				try {
					urlFile = new URL(imageName);
				} catch (java.net.MalformedURLException e) {
                    System.out.println(imageName + ": Malformed URL !");
				}
            }
            if(urlFile!=null){
                return new ImageIcon(urlFile);
            }
        }
        return null;
    }

    protected void createFileMenu(JMenuBar menu, JToolBar toolBar) {
        JMenu fileMenu = new JMenu("File");
        fileMenu.setMnemonic(KeyEvent.VK_F);

        JMenuItem menuItem = null;
        JButton button = null;

        Action refreshAction = new RefreshAction("Refresh the tree", createIcon("Reload.png"), "Refreshes the tree", new Integer(KeyEvent.VK_R));
        Action quitAction = new QuitAction("Quit", createIcon("Power.png"), "Quit the application", new Integer(KeyEvent.VK_Q));
        
        // Add to the menubar

        menuItem = new JMenuItem(refreshAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, Event.CTRL_MASK));
        fileMenu.add(menuItem);

        menuItem = new JMenuItem(quitAction);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, Event.ALT_MASK));
        fileMenu.add(menuItem);

        // Adding to the toolbar

        button = new JButton(quitAction);
        button.setText("");
        toolBar.add(button);

        button = new JButton(refreshAction);
        button.setText("");
        toolBar.add(button);

        menu.add(fileMenu);
    }
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * Default constructor
     */
    public BrowserImpl(){
       
        JFrame frm = new JFrame("Fractal Browser");
        
        tree_ = new DynamicTree();
        
        DefaultViewPanel viewPanel = new DefaultViewPanel();
        JMenuBar menuBar = new JMenuBar();
        JToolBar toolBar = new JToolBar();        
        createFileMenu(menuBar, toolBar);
        
        tree_.setTargetPanel(viewPanel);
        tree_.setJMenuBar(menuBar);
        tree_.setJToolBar(toolBar);
        
        DefaultStatusBar statusBar = new DefaultStatusBar();
        tree_.setStatusBar(statusBar);

        frm.setJMenuBar(menuBar);
        frm.getContentPane().setLayout(new BorderLayout());
        frm.getContentPane().add(toolBar,BorderLayout.NORTH);
        frm.getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, new DefaultTreePanel(tree_), viewPanel), BorderLayout.CENTER);
        frm.getContentPane().add(statusBar,BorderLayout.SOUTH);

        frm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frm.pack();

        // Centers the frame
        Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        frm.setLocation((screenSize.width - frm.getWidth()) / 2, (screenSize.height - frm.getHeight()) / 2);

        frm.setVisible(true); 
    }
   
    // ==================================================================
    //
    // Public methods for interface BrowserAttributes.
    //
    // ==================================================================
   
    public void setConfigFiles (String configFiles) {
        if(configFiles != null){
            StringTokenizer st = new StringTokenizer(configFiles,": ");
            List l = new Vector();
            while (st.hasMoreTokens()) { 
                String theFile = st.nextToken();
                configFiles_.add(theFile);
                l.add(theFile);
            }
            tree_.addBrowserProperty((String[])l.toArray(new String[0]));
            tree_.setCurrentRole(new String[]{"user"});
        }
    }
    
    public String getConfigFiles () {
        StringBuffer sb = new StringBuffer();
        Iterator it = configFiles_.iterator();
        while(it.hasNext()){
            sb.append((String)it.next() + " ");
        }
        return sb.toString();
    }
 
    // ==================================================================
    //
    // Public methods for interface BindingController.
    //
    // ==================================================================
  
    public void bindFc(String clientItfName, Object sItf)
        throws
            NoSuchInterfaceException,
            IllegalBindingException,
            IllegalLifeCycleException {
        if (clientItfName.startsWith(FC_APPL)) {
            elements_.put(clientItfName, sItf);
            tree_.addEntry(getName((Component)sItf), sItf,1);
        }
    }

    public String[] listFc() {
        return (String[])elements_.keySet().toArray(new String[elements_.size()]);
    }

    public Object lookupFc(String clientItfName)
        throws NoSuchInterfaceException {
        if (clientItfName.startsWith(FC_APPL)) {
            return elements_.get(clientItfName);
        }
        return null;
    }

    public void unbindFc(String clientItfName)
        throws
            NoSuchInterfaceException,
            IllegalBindingException,
            IllegalLifeCycleException {
        if (clientItfName.equals(FC_APPL)) {
            elements_.remove(clientItfName);   
        }
    }

    // ==================================================================
    //
    // Internal classes.
    //
    // ==================================================================

    protected class RefreshAction extends AbstractAction {
    
        public RefreshAction(String nom, ImageIcon image, String desc, Integer mnemonic) {
            super(nom, image);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            BrowserImpl.this.tree_.refreshAll();
        }
    }

    protected class QuitAction extends AbstractAction {
    
        public QuitAction(String nom, ImageIcon image, String desc, Integer mnemonic) {
            super(nom, image);
            putValue(SHORT_DESCRIPTION, desc);
            putValue(MNEMONIC_KEY, mnemonic);
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            System.exit(0);
        }
    }

}