/*====================================================================
 
 Objectweb Browser framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: DefaultFractalContext.java,v 1.2 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.context;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.plugins.fractal.api.FractalContext;

/**
 * Basic implementation of a Fractal context.
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.2
 */
public abstract class DefaultFractalContext 
           implements Context, 
                      Wrapper, 
                      FractalContext 
{
    
    //==================================================================
    //
    // No internal state.
    //
    //==================================================================
    
    /** The associated object. */
    protected Object object_;
    
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================
    
    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================
    
    //==================================================================
    //
    // Public methods for Context interface.
    //
    //==================================================================
    
    /**
     * Overriding methods
     * @see org.objectweb.util.browser.api.Context#getEntries()
     */
    public Entry[] getEntries() {
        LifeCycleController lcc = null;
        try {
             lcc = Fractal.getLifeCycleController(NameProvider.singleton.getComponent(object_));
        } catch (NoSuchInterfaceException e2) {
            try {
                return getStartedEntries();
            }  catch (Exception e1) {
                return getStoppedEntries();
            }
        }
        if (lcc.getFcState().equals(LifeCycleController.STARTED))
            return getStartedEntries();
        else
            return getStoppedEntries();
    }
    
    //==================================================================
    //
    // Public methods for FractalContext interface.
    //
    //==================================================================
    
    /**
     * Called when the Fractal component is started.
     * @return An array of entries contained into the Context.
     */
    public abstract Entry[] getStartedEntries() ;
    
    /**
     * Called when the Fractal component is stopped.
     * @return An array of entries contained into the Context.
     */
    public Entry[] getStoppedEntries() {
        return new Entry[0];
    }
    
    //==================================================================
    //
    // Public methods for Wrapper interface.
    //
    //==================================================================
    
    /**
     * Sets the wrapped object.
     * @param object The object to wrap.
     */
    public void setWrapped(Object object) {
        object_ = object;
    }
    
    /**
     * Gets the wrapped object.
     * @return The wrapped object.
     */
    public Object getWrapped() {
        return object_;
    }
}