/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): Romain Rouvoy______________________________________.
 
 ---------------------------------------------------------------------
 $Id: ClientInterfaceIconProvider.java,v 1.1 2004/04/14 09:43:14 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.icon;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.plugins.fractal.context.ClientInterfaceWrapper;

/**
 * Provides a specific icon for InterfaceReference if this one is :
 * <br>
 * <li>a client interface.</li>
 * <li>not bound.</li>
 * </br>
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ClientInterfaceIconProvider 
  implements IconProvider
{
    
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================
    
    /** The different icons. */
    protected Icon notBoundMandatoryIcon, boundIcon;
    
    // ==================================================================
    //
    // Contructors.
    //
    // ==================================================================
    
    /** Empty constructor. */
    public ClientInterfaceIconProvider() {
        boundIcon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("FcClient.png"));
        notBoundMandatoryIcon = new ImageIcon(Thread.currentThread().getContextClassLoader().getResource("FcClient2.png"));
    }
    
    // ==================================================================
    //
    // Public methods for IconProvider interface.
    //
    // ==================================================================
    
    /**
     * Provides the appropriate icon.
     */
    public Icon newIcon(Object obj) {
        ClientInterfaceWrapper ciw = (ClientInterfaceWrapper) obj;
        Interface ir = ciw.getItf();
        Component ci = ir.getFcItfOwner();
        InterfaceType interfaceType = (InterfaceType)ir.getFcItfType();
        if(interfaceType != null && interfaceType.isFcCollectionItf()){
            if(!interfaceType.getFcItfName().equals(ir.getFcItfName()))
                return boundIcon;
            else
                return notBoundMandatoryIcon;
        } else if (interfaceType != null){
            Interface bindInterface = null;
            try{
                BindingController bc = Fractal.getBindingController(ci);
                bindInterface = (Interface)bc.lookupFc(((InterfaceType)ir.getFcItfType()).getFcItfName());
            }catch(Exception e) {
                e.printStackTrace();
                System.err.println("Icon Provider Error : " + e.getMessage());   
            }             
            return bindInterface==null ? notBoundMandatoryIcon : boundIcon;            
        }
        return notBoundMandatoryIcon;
    }
    
}