/*====================================================================
 
 Objectweb Browser Framework
 Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
 Contact: openccm@objectweb.org
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA
 
 Initial developer(s): Jerome Moroy.
 Contributor(s): ______________________________________.
 
 ---------------------------------------------------------------------
 $Id: DefaultFractalViewPanel.java,v 1.2 2004/04/14 09:43:15 rouvoy Exp $
 ====================================================================*/

package org.objectweb.util.browser.plugins.fractal.panel;

import javax.swing.JPanel;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.browser.api.Panel;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.core.panel.WhitePanel;
import org.objectweb.util.browser.plugins.fractal.api.*;
import org.objectweb.util.browser.plugins.fractal.context.NameProvider;

/**
 * Specialization of the view panel for entities having life cycle properties.
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public abstract class DefaultFractalViewPanel 
           implements Panel, 
                      FractalViewPanel 
{
    //==================================================================
    //
    // Internal state.
    //
    //==================================================================
    
    /** A white panel. */
    protected final Panel whitePanel_ = new WhitePanel();
    
    /** Boolean which indicates if the component is strated or not. */
    protected boolean isStarted_ = true;
    
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================
    
    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================
    
    //==================================================================
    //
    // Public methods for TreeView interface.
    //
    //==================================================================
    
    /**
     * @see org.objectweb.util.browser.api.ViewPanel#preInitialize(org.objectweb.util.browser.api.TreeView)
     */
    public void selected(TreeView treeView) {
        LifeCycleController lcc = null;
        try{
            lcc = Fractal.getLifeCycleController(NameProvider.singleton().getComponent(treeView.getSelectedObject()));
        } catch(NoSuchInterfaceException e) {
            try {
                selectedStarted(treeView);
            } catch (Exception e1) {
                isStarted_ = false;
                selectedStopped(treeView);
            }
        }
        if(lcc.getFcState().equals(LifeCycleController.STARTED)){
            isStarted_ = true;        
            selectedStarted(treeView);
        } else {
            isStarted_ = false;
            selectedStopped(treeView);
        }
    }
    
    /**
     * @see org.objectweb.util.browser.api.ViewPanel#getPanel()
     */
    public JPanel getPanel(){
        if(isStarted_){
            try{
                return getStartedPanel();
            } catch (Exception e) {
                return getStoppedPanel();
            }
        } else {
            return getStoppedPanel();
        } 
    }
    
    /**
     * @see org.objectweb.util.browser.api.ViewPanel#postInitialize(org.objectweb.util.browser.api.TreeView)
     */
    public void unselected(TreeView treeView) {
        LifeCycleController lcc = null;
        try{
            lcc = Fractal.getLifeCycleController(NameProvider.singleton().getComponent(treeView.getSelectedObject()));
        } catch(NoSuchInterfaceException e) {
            try {
                unselectedStarted(treeView);
            } catch (Exception e1) {
                unselectedStopped(treeView);
            }
        }
        if(lcc.getFcState().equals(LifeCycleController.STARTED))
            unselectedStarted(treeView);
        else
            unselectedStopped(treeView);
    }
    
    //==================================================================
    //
    // Public methods for FractalTreeView interface.
    //
    //==================================================================
    
    public abstract void selectedStarted(TreeView treeView);
    
    public void selectedStopped(TreeView treeView) { 
    }
    
    public abstract JPanel getStartedPanel();
    
    public JPanel getStoppedPanel() { 
        return whitePanel_.getPanel();
    }
    
    public abstract void unselectedStarted(TreeView treeView);
    
    public void unselectedStopped(TreeView treeView){ 
    }
    
}