/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.core.common.ClassResolver;

/**
 * Provides an icon to represent <code>java.lang.reflect.Field</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class FieldIconProvider 
    implements IconProvider {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The icons to use for public field. */
    protected static Icon publicField_, publicStaticField_, publicFinalField_, publicStaticFinalField_; 
    
    /** The icons to use for protected field. */
    protected static Icon protectedField_, protectedStaticField_, protectedFinalField_, protectedStaticFinalField_;
    
    /** The icons to use for package field. */
    protected static Icon packageField_, packageStaticField_, packageFinalField_, packageStaticFinalField_;

    /** The icons to use for private field. */
    protected static Icon privateField_, privateFinalField_;

    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    /**
     * Default constructor
     */
    public FieldIconProvider(){
        publicField_ = new ImageIcon(ClassResolver.getResource("publicField.png"));
        publicStaticField_ = new ImageIcon(ClassResolver.getResource("publicStaticField.png"));
        publicFinalField_ = new ImageIcon(ClassResolver.getResource("publicFinalField.png"));
        publicStaticFinalField_ = new ImageIcon(ClassResolver.getResource("publicStaticFinalField.png"));
        protectedField_ =  new ImageIcon(ClassResolver.getResource("protectedField.png"));
        protectedStaticField_ = new ImageIcon(ClassResolver.getResource("protectedStaticField.png"));
        protectedFinalField_ = new ImageIcon(ClassResolver.getResource("protectedFinalField.png"));
        protectedStaticFinalField_ = new ImageIcon(ClassResolver.getResource("protectedStaticFinalField.png"));
        privateField_ =  new ImageIcon(ClassResolver.getResource("privateField.png"));
        privateFinalField_ =  new ImageIcon(ClassResolver.getResource("privateFinalField.png"));
        packageField_ =  new ImageIcon(ClassResolver.getResource("packageField.png"));
        packageStaticField_ = new ImageIcon(ClassResolver.getResource("packageStaticField.png"));
        packageFinalField_ = new ImageIcon(ClassResolver.getResource("packageFinalField.png"));
        packageStaticFinalField_ = new ImageIcon(ClassResolver.getResource("packageStaticFinalField.png"));        
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for IconProvider interface.
    //
    //==================================================================

    /**
     * Return an <code>javax.swing.Icon</code> according to the given object.
     *
     * @param object The object to compute the associated icon.
     * 
     * @return The icon according to the given object.
     */
    public Icon 
    newIcon(Object object){
        Field field_ = (Field)object;
        int modifier = field_.getModifiers();
        if (Modifier.isPrivate(modifier)){
            if (Modifier.isFinal(modifier))
                return privateFinalField_;
            else
                return privateField_;                
        } else if (Modifier.isProtected(modifier)){
            if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return protectedStaticFinalField_;
            else if (Modifier.isStatic(modifier))
                return protectedStaticField_;
            else if (Modifier.isFinal(modifier))
                return protectedFinalField_;
            else
                return protectedField_;                
        } else if (Modifier.isPublic(modifier)){
            if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return publicStaticFinalField_;
            else if (Modifier.isStatic(modifier))
                return publicStaticField_;
            else if (Modifier.isFinal(modifier))
                return publicFinalField_;
            else
                return publicField_;                
        } else if (!Modifier.isPrivate(modifier) && !Modifier.isProtected(modifier) && !Modifier.isPublic(modifier)) {
            if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return packageStaticFinalField_;
            else if (Modifier.isStatic(modifier))
                return packageStaticField_;
            else if (Modifier.isFinal(modifier))
                return packageFinalField_;
            else
                return packageField_;                        
        } else {
            return null;   
        }
    }
    
}