/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Comparator;

/**
 * This class compares two <code>Field</code> objects 
 * or two <code>Method</code> objects, or two <code>Class</code> objects 
 * lexicographically.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class MemberComparator
    implements Comparator {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    protected int compare(String s1, String s2){
        return s1.compareTo(s2);
    }

    protected int compare(Class c1, Class c2){
        return compare(c1.getName(), c2.getName());
    }

    protected int compare(Field f1, Field f2){
        return compare(f1.getName(), f2.getName());
    }
        
    protected int compare(Method m1, Method m2){
        return compare(m1.getName(), m2.getName());
    }

    //==================================================================
    //
    // Public method for Comparator interface.
    //
    //==================================================================

    public int compare(Object o1, Object o2) {
        if(o1!=null && o2!=null){
            if(o1 instanceof Class && o2 instanceof Class)
                return compare((Class)o1,(Class)o2);
            else if(o1 instanceof Field && o2 instanceof Field)
                return compare((Field)o1,(Field)o2);
            else if(o1 instanceof Method && o2 instanceof Method)
                return compare((Method)o1,(Method)o2);
            else
                return 0;
        }
        return 0;
    }

}
    