/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.common;

/** The Java API's imports */
import java.io.InputStream;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * This class is used to resolve class from the ClassLoader. It contains the jar loaded from the config files. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ClassResolver {

    /** The current ClassLoader (Default value : The thread's classLoader) */
    protected static URLClassLoader current_ =
        (URLClassLoader) Thread.currentThread().getContextClassLoader();

    /**
     * Adds the given URLs into the context (ClassLoader)
     * @param urls The URLs to search for classes and resources
     */
    public static void addContext(URL[] urls) {
        if (urls != null) {
            URLClassLoader newCL = URLClassLoader.newInstance(urls, current_);
            current_ = newCL;
            Thread.currentThread().setContextClassLoader(current_);
        }
    }

    /**
     * Returns the Class object associated with the class or interface with the given string name.
     * @param className The fully qualified name of the desired class
     * @return A class object representing the desired class
     * @exception ClassNotFoundException If the class cannot be located
     * @see java.lang.ClassLoader
     */
    public static Class resolve(String className)
        throws ClassNotFoundException {
        return Class.forName(className, true, current_);
    }
    
    /**
     * Finds the resource with the given name.
     * @param name The resource name
     * @return A URL object for reading the resource, or null if the resource could not be found or the invoker doesn't have adequate privileges to get the resource.
     */
    public static URL getResource(String name) {
        return current_.getResource(name);
    }
        
    /**
     * Returns an input stream for reading the specified resource.
     * @param name The resource name
     * @return An input stream for reading the resource, or null  if the resource could not be found
     */
    public static InputStream getResourceAsStream(String name) {
        return current_.getResourceAsStream(name);  
    }

}
