/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.dnd;

import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.core.api.DropActionFactory;

/**
 * Basic implementation for DropAction factories
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultDropActionFactory 
    implements DropActionFactory {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** A description of the action */
    protected String label_ = null;

    /** The class to instantiate */
    protected Class class_ = null;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for DropActionFactory interface.
    //
    //==================================================================

    /* 
     * @see org.objectweb.util.browser.api.DropActionFactory#newDropAction(java.lang.Object)
     */
    public DropAction newDropAction(Object object) {
        if (class_ != null) {
            try {
                return (DropAction) class_.newInstance();
            } catch (java.lang.InstantiationException e) {
                System.out.println(class_.getName() + " : Instanciation exception");
            } catch (java.lang.IllegalAccessException e) {
                System.out.println(class_.getName() + " : Illegal access exception");
            }
        }
        return null;
    }

    /* 
     * @see org.objectweb.util.browser.api.DropActionFactory#setClassName(java.lang.Class)
     */
    public void setClassName(Class theClass) {
        class_ = theClass;
    }

    /*
     * @see org.objectweb.util.browser.api.DropActionFactory#setLabel(java.lang.String)
     */
    public void setLabel(String label){
        label_ = label;    
    }
    
	/* 
	 * @see org.objectweb.util.browser.api.DropActionFactory#getLabel()
	 */
	public String getLabel() {
		return label_;
	}

}