/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** The AdminConsole's imports */
import java.util.Enumeration;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.ContextContainer;

/**
 * Basic implementation of the Context interface
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */
public class DefaultContextContainer 
    extends DefaultContext 
    implements ContextContainer {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Empty contstructor
     */
    public DefaultContextContainer() {
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Additional public methods for class DefaultContextContainer.
    //
    // ==================================================================

    /**
     * Returns the entry identify by the given name
     * @param name The key of the entry
     * @return The associated entry or null.
     */
    public Entry getLocalEntry(String name) {
        for (Enumeration e = elts_.elements(); e.hasMoreElements();) {
            Entry entry = (Entry) e.nextElement();
            if (entry.getName().toString().equals(name)) {
                return entry;
            }
        }
        return null;
    }

    /**
     * Returns the value of the entry identify by the given name
     * @param name The key of the entry
     * @return The value of the associated entry or null.
     */
    public Object getLocalValue(String name){
        Entry entry = getLocalEntry(name);
        return entry!=null ? entry.getValue() : null;    
    }

    /**
     * Adds an object into the container.
     * If an entry with the same id already exists in the container, this one is replaced by the new value. 
     *
     * This method does not modify the given id and the given object, But it modifies the target context.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The object to add
     * 
     */
    public void addEntry(String id, Object object) {
        if (object != null) {
            removeEntry(id);
            elts_.add(createEntry(object, id));
        }
    }

    /**
     * Removes the object corresponding to the given id from the context
     * This method modifies the target context.
     * @param id The name of the entry to be removed.
     */
    public void removeEntry(String id) {
        Entry oldEntry = getLocalEntry(id);
        if (oldEntry != null) {
            elts_.remove(oldEntry);
        }
    }

    /**
     * Removes all of the elements from this context.
     */
    public void clear(){
        elts_.clear();
    }
}
