/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** The console's imports  */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.api.Decoder;
import org.objectweb.util.browser.core.api.SpecificContextFactory;

/**
 * Basic implementation for Context factories.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultContextFactory implements SpecificContextFactory {

    /** The name of the class to instanciate */
    protected Class class_ = null;

    /** The decoder */
    protected Decoder decoder_ = null;

    /**
     * Default Constructor
     * @param decoder The first link of the chain of decoder
     */
    public DefaultContextFactory(Decoder decoder) {
        decoder_ = decoder;
    }

    /**
     * Fix the class to instanciate. 
     * This class has to implemente Context and Wrapper interfaces. 
     * 
     * @param theClass The class to instanciate
     */
    public void setClassName(Class theClass) {
        class_ = theClass;
    }

    /**
     * Creates a new {@link Context Context} associated to an object.
     *
     * @return The created {@link Context Context}.
     */
    public Context newOWContext(Object object) {
        if (class_ != null) {
            try {
                Wrapper wrapper = (Wrapper) class_.newInstance();
                wrapper.setWrapped(object);
                if (decoder_ != null) {
                    ContextDecoderManager interceptor = new ContextDecoderManager();
                    interceptor.setDecoder(decoder_);
                    interceptor.setDelegate((Context) wrapper);
                    return interceptor;
                } else {
                    return (Context) wrapper;
                }
            } catch (InstantiationException e) {
                e.printStackTrace();
            } catch (IllegalAccessException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

}
