/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies.
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;


/**
 * This application wraps the BES idl2java command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class IdlToJava
     extends IdlToJavaApplication
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** ORB properties. */
    Properties props_ = null;

    /** Arguments delimiter. */
    private String arg_delimiter_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /** The default constructor */
    public IdlToJava()
    {
        if ( System.getProperty("file.separator").compareTo("\\") == 0 )
            arg_delimiter_ = "\"";
        else
            arg_delimiter_ = "";
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Get the Orbacus jidl command.
     * 
     * @return The full path to the jidl command.
     */
    private void
    loadProperties()
    {
        InputStream input = null;
        
        props_ = new Properties();
        
        // Load build.properties user file
        input = Thread.currentThread().getContextClassLoader().getResourceAsStream("build.properties");
        try
        {
            props_.load(input);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Get the Orbacus jidl command.
     * 
     * @return The full path to the jidl command.
     */
    private String
    getCommand()
    {
        return props_.getProperty("ORB.home.dir") + "/bin/idl2java ";
    }

    /**
     * Get the Orbacus jidl arguments.
     * 
     * @return A string with all arguments.
     */
    private String
    getArguments()
    {
        String args = null,
               key = null,
               value = null;
        String[] includes = null;
        Map mappings = null;
        Set mapping_keys = null;
       
        // Default arguments
        args = "-strict ";
        
        // Add the destination directory
        args += "-root_dir " + arg_delimiter_ + getDestinationDirectory() + arg_delimiter_ + " ";
        
        // Add Include directory list
        includes = getIncludeDirectories();
        for (int i=0; i<includes.length; i++)
        {
            args += arg_delimiter_ + "-I" + includes[i] + arg_delimiter_ + " ";
        }

        // Add Package mappings
        mappings = getPackageMappings();
        mapping_keys = getPackageMappings().keySet();
        for (Iterator it=mapping_keys.iterator(); it.hasNext(); )
        {
            key = (String) it.next();
            value = (String) mappings.get(key);
            args += "-idl2package ::" + key + " " + value + " ";
        }

        // Add the file to parse
        args += arg_delimiter_ + getInputFile() + arg_delimiter_;
        
        return args;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
      * Starts the main function.
      *
      * @param args The command line arguments.
      *
      * @return The status of the application.
      */
    public int
    start(String[] args)
    {
        Process process = null;
        Redirector redirector = null;
        
        parse_cmdline(args);
        loadProperties();
        
        try
        {
            process = Runtime.getRuntime().exec( getCommand() + getArguments() );
            
            redirector = new Redirector(process);
            redirector.redirectOutput();
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        try
        {
            process.waitFor();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        
        return process.exitValue();
    }
    
    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    public static void
    main(String[] args)
    {
        IdlToJava app = new IdlToJava();
        app.runMain(args);
    }
}
