/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): offroy ________________________.
Contributor(s): ______________________________________.

====================================================================*/
/*
 * Created on 23 mai 2003 by jerome OFFROY (offroy@lifl.fr)
 *
 */

package org.omg.lifl.eclipse.plugin.project.OpenCCM.utils;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.omg.lifl.eclipse.plugin.project.utils.AttributeManager;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.CheckGroup;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooser;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooserCheckRadioButtonSetter;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooserCheckTextSetter;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.DirectoryChooserNoCheckTextSetter;
import org.omg.lifl.eclipse.plugin.project.utils.SWT.SetterGroupPreference;

/**
 * This class deals with OpenCCM build.properties configuration file
 *      - store preferences
 *      - create SWT widget to change preferences
 *
 * @author LIFL
 *
 * @version 0.1
 */
public class OpenCCMConfigChooser {

    private String groupTitle;

    private Group group_ORB_Chooser;
    private CheckGroup checkGroup;
    private String _ORBName;

    private SetterGroupPreference _ORBpreference;
    private SetterGroupPreference _InstallOpenCCM;
    private SetterGroupPreference _TransactionpreferenceHome;
    private SetterGroupPreference _TransactionpreferenceJar;
    private SetterGroupPreference _NSDirPreference;
    private SetterGroupPreference _PreprocessorPreference;
    private SetterGroupPreference _JavaccPreference;
    private SetterGroupPreference _CompilationOptionDebugPreference;
    private SetterGroupPreference _CompilationOptionDeprecationPreference;
    private SetterGroupPreference _CompilationOptionOptimizePreference;
    
    /**
     * Construct the preference needed for configuring the 
     * OpenCCM build.properties configuration file
     * @param elem the element which contains needed configuration
     */
    public OpenCCMConfigChooser(IConfigurationElement elem) {
        groupTitle = AttributeManager.getAttribute(elem, "ORBConfigGroupLabel");

        // get the ORB Name
        _ORBName = AttributeManager.getAttribute(elem, "ORBName");

        // to set ORB Path
        _ORBpreference = AttributeManager.getAllAttribute(elem, "OB");

        // to set The directory where the OpenCCM Platform will be installed.
        _InstallOpenCCM =
            AttributeManager.getAllAttribute(elem, "InstallOpenCCM");

        // to set Transaction Home directory and Jar directory
        _TransactionpreferenceHome =
            AttributeManager.getAllAttribute(elem, "TransactionHome");
        _TransactionpreferenceJar =
            AttributeManager.getAllAttribute(elem, "TransactionJar");
        _NSDirPreference = AttributeManager.getAllAttribute(elem, "NShomeDir");
        _PreprocessorPreference =
            AttributeManager.getAllAttribute(elem, "CppDir");
        _JavaccPreference = AttributeManager.getAllAttribute(elem, "JavaccDir");
        //TODO change class to store preference (no String but boolean )
        _CompilationOptionDebugPreference =
            AttributeManager.getAllAttribute(elem, "CompilationOptionDebug");
        _CompilationOptionDeprecationPreference =
            AttributeManager.getAllAttribute(
                elem,
                "CompilationOptionDeprecation");
        _CompilationOptionOptimizePreference =
            AttributeManager.getAllAttribute(elem, "CompilationOptionOptimize");
    }

    /**
     * create a SWT group which a check
     * to set OpenCCM build.properties configuration file
     * @param composite the parent Composite 
     */
    public void viewToSetCCMConfig(Composite composite) {
        checkGroup = new CheckGroup(composite, groupTitle);

        Group parent_group = new Group(checkGroup.getGroup(), SWT.NONE);
        parent_group.setLayout(new RowLayout(SWT.VERTICAL));
        checkGroup.setSubGroup(parent_group);

        // area to choose the location of the ORB
        DirectoryChooser dcORB =
            new DirectoryChooserNoCheckTextSetter(parent_group, _ORBpreference);

        // area to choose The directory where the OpenCCM Platform will be installed.
        DirectoryChooser dcInstallOpenCCM =
            new DirectoryChooserCheckTextSetter(parent_group, _InstallOpenCCM);

        // area to choose the location of the Transaction Home Directory and jar directory
        DirectoryChooser dcTransactionHome =
            new DirectoryChooserCheckTextSetter(
                parent_group,
                _TransactionpreferenceHome);
        DirectoryChooser dcTransactionJar =
            new DirectoryChooserCheckTextSetter(
                parent_group,
                _TransactionpreferenceJar);

        // area to choose the location of the NS 
        DirectoryChooser dcNSDir =
            new DirectoryChooserCheckTextSetter(parent_group, _NSDirPreference);

        // area to choose the location of the Preprocessor to use
        DirectoryChooser dcCppDir =
            new DirectoryChooserCheckTextSetter(
                parent_group,
                _PreprocessorPreference);

        // area to choose the location of the JavaCC 
        DirectoryChooser dcJavaCC =
            new DirectoryChooserCheckTextSetter(
                parent_group,
                _JavaccPreference);

        // area to choose the ant compilation option Debug value
        DirectoryChooser dcCompilDebug =
            new DirectoryChooserCheckRadioButtonSetter(
                parent_group,
                _CompilationOptionDebugPreference);
        // area to choose the ant compilation option Deprecation value
        DirectoryChooser dcCompilDeprecation =
            new DirectoryChooserCheckRadioButtonSetter(
                parent_group,
                _CompilationOptionDeprecationPreference);
        // area to choose the ant compilation option Optimize value
        DirectoryChooser dcCompilOptimize =
            new DirectoryChooserCheckRadioButtonSetter(
                parent_group,
                _CompilationOptionOptimizePreference);
    }

    /**
     * @return a the ORB's name 
     */
    public String get_ORBName() {
        return _ORBName;
    }

    /**
     * @param string to set ORB's name 
     */
    public void set_ORBName(String string) {
        _ORBName = string;
    }

    /**
     * @return the main SWT CheckGroup
     */
    public CheckGroup getCheckGroup() {
        return checkGroup;
    }

    /**
     * @param group is used to set main SWT CheckGroup 
     */
    public void setCheckGroup(CheckGroup group) {
        checkGroup = group;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * the ORB's location
     */
    public SetterGroupPreference get_ORBpreference() {
        return _ORBpreference;
    }

    /**
     * @param preference SetterGroupPreference to set ORB's location
     */
    public void set_ORBpreference(SetterGroupPreference preference) {
        _ORBpreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * the Transactional Service home directory
     */
    public SetterGroupPreference get_TransactionpreferenceHome() {
        return _TransactionpreferenceHome;
    }

    /**
     * @param preference SetterGroupPreference to set 
     * Transactional Service Home directory
     */
    public void set_TransactionpreferenceHome(SetterGroupPreference preference) {
        _TransactionpreferenceHome = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * Transactional Service Jar location
     */
    public SetterGroupPreference get_TransactionpreferenceJar() {
        return _TransactionpreferenceJar;
    }

    /**
     * @param preference SetterGroupPreference to set
     * Transactional Service Jar location
     */
    public void set_TransactionpreferenceJar(SetterGroupPreference preference) {
        _TransactionpreferenceJar = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * NameService's location
     */
    public SetterGroupPreference get_NSDirPreference() {
        return _NSDirPreference;
    }

    /**
     * @param preference SetterGroupPreference to set
     * NameService's location
     */
    public void set_NSDirPreference(SetterGroupPreference preference) {
        _NSDirPreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * OpenCCM Ant Compilation Option Debug 
     */
    public SetterGroupPreference get_CompilationOptionDebugPreference() {
        return _CompilationOptionDebugPreference;
    }

    /**
     * @param preference  SetterGroupPreference to set
     * OpenCCM Ant Compilation Option Debug
     */
    public void set_CompilationOptionDebugPreference(SetterGroupPreference preference) {
        _CompilationOptionDebugPreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * OpenCCM Ant Compilation Option Deprecation
     */
    public SetterGroupPreference get_CompilationOptionDeprecationPreference() {
        return _CompilationOptionDeprecationPreference;
    }

    /**
     * @param preference SetterGroupPreference to set
     * OpenCCM Ant Compilation Option Deprecation
     */
    public void set_CompilationOptionDeprecationPreference(SetterGroupPreference preference) {
        _CompilationOptionDeprecationPreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * OpenCCM Ant Compilation Option Optimize
     */
    public SetterGroupPreference get_CompilationOptionOptimizePreference() {
        return _CompilationOptionOptimizePreference;
    }

    /**
     * @param preference SetterGroupPreference to set
     * OpenCCM Ant Compilation Option Optimize
     */
    public void set_CompilationOptionOptimizePreference(SetterGroupPreference preference) {
        _CompilationOptionOptimizePreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * JavaCC's location
     */
    public SetterGroupPreference get_JavaccPreference() {
        return _JavaccPreference;
    }

    /**
     * @param preference SetterGroupPreference to set
     * JavaCC's location
     */
    public void set_JavaccPreference(SetterGroupPreference preference) {
        _JavaccPreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * Preprocessor's location
     */
    public SetterGroupPreference get_PreprocessorPreference() {
        return _PreprocessorPreference;
    }

    /**
     * @param preference SetterGroupPreference to set
     * Preprocessor's location
     */
    public void set_PreprocessorPreference(SetterGroupPreference preference) {
        _PreprocessorPreference = preference;
    }

    /**
     * @return a SetterGroupPreference which contains the value of
     * the directory where the OpenCCM Platform will be installed.
     */
    public SetterGroupPreference get_InstallOpenCCM() {
        return _InstallOpenCCM;
    }

    /**
     * @param preference SetterGroupPreference to set
     * the directory where the OpenCCM Platform will be installed.
     */
    public void set_InstallOpenCCM(SetterGroupPreference preference) {
        _InstallOpenCCM = preference;
    }
}
