/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/
package org.coach.actor.commandline;

/** The Command Lines framework's imports */
import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultOptionBase;

/**
 * Class representing the option to set the default actor configuration file to interpret.<br>
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultActorOption extends DefaultOptionBase {

    /** Reference to the Default Application. */
    protected DefaultApplication application_;

    /**
     * The constructor with the owner Default Application.
     * @param application Reference to the Default Application.
     */
    public DefaultActorOption(DefaultApplication application) {
        // Calls the DefaultOptionBase constructor.
        super();
        setLabels(new String[] { "--defaultActor" });
        setArguments(new String[] { "file" });
        setDescription(new String[] { "Default actor XML file (You don't have to use this option !)" });

        // Inits internal state.
        application_ = application;
    }

    // ==================================================================
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     * @param iterator The command line argument iterator.
     */
    public void consume(org.objectweb.util.cmdline.api.Iterator iterator) {
        checkAlreadySet(iterator);
        String file = consumeArgument(iterator);
        application_.setDefaultActorConfigFile(file);
    }

}
