/*
 * TraceViewer.java
 *
 * Created on March 17, 2003, 8:05 PM
 */

package org.coach.tracing.server.viewer;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.lang.reflect.*;
import java.awt.print.*;
import org.coach.tracing.server.EventDataBase;
import org.coach.tracing.server.EventRecord;

/**
 *
 * @author  harold
 */
public class TraceViewerFrame extends javax.swing.JFrame {
    static ScaleManager scaleManager = ScaleManager.getScaleManager();
    Diagram diagramPanel;
    private long prevStart = -1;
    private LinkedList browsers = new LinkedList();    
    private ActionListener updateAction;
    private javax.swing.Timer updateTimer;
    private long eventCount = 0;
    private EventDataBase eventDB;
    
    /** Creates new form TraceViewer */
    public TraceViewerFrame() {       
        diagramPanel = new Diagram(this);
        initComponents();
    }
    
    public void setEventDB(EventDataBase db) {
        eventDB = db;
        diagramPanel.setEventDB(eventDB);
        IdentityNode.setEventDB(eventDB);
        ThreadNode.clear();
        EventRecord.setEventDB(eventDB);
        int units = (int)(scaleManager.getScale() * Diagram.space_between);
        int range = getHeight() / units;
        diagramPanel.setRange(range);    
        updateAction = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                long newEventCount = eventDB.getEventCount();
                if (newEventCount != eventCount) {
                    eventCount = newEventCount;
                    diagramPanel.updateStatus(eventCount);
                    diagramPanel.repaintEvents();
                }
            }
        };        
        updateTimer = new javax.swing.Timer(1000, updateAction);
        updateTimer.start();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        viewModebuttonGroup = new javax.swing.ButtonGroup();
        statusPanel = new javax.swing.JPanel();
        eventsLabel = new javax.swing.JLabel();
        eventsTextField = new javax.swing.JTextField();
        messagesLabel = new javax.swing.JLabel();
        messagesTextField = new javax.swing.JTextField();
        unmatchedEventsLabel = new javax.swing.JLabel();
        unmatchedEventsTextField = new javax.swing.JTextField();
        buttonPanel = new javax.swing.JPanel();
        clearButton = new javax.swing.JButton();
        uncolorButton = new javax.swing.JButton();
        trailLabelButton = new javax.swing.JButton();
        zoomInButton = new javax.swing.JButton();
        zoomOutButton = new javax.swing.JButton();
        unhideButton = new javax.swing.JButton();
        identitiesButton = new javax.swing.JButton();
        diagramMainPanel = new javax.swing.JPanel();
        diagramVerticalScrollBar = new javax.swing.JScrollBar();
        diagramScrollPanel = new javax.swing.JScrollPane();
        diagramScrollPanel.getViewport().add(diagramPanel, null);
        diagramScrollPanel.setFont(scaleManager.getFont());
        diagramScrollPanel.getHorizontalScrollBar().setUnitIncrement((int)(scaleManager.getScale() * IdentityGui.min_hsize));
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        connectMenuItem = new javax.swing.JMenuItem();
        openMenuItem = new javax.swing.JMenuItem();
        saveAsMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        optionsMenu = new javax.swing.JMenu();
        threadViewRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        objectViewRadioButtonMenuItem = new javax.swing.JRadioButtonMenuItem();
        helpMenu = new javax.swing.JMenu();
        contentsMenuItem = new javax.swing.JMenuItem();
        aboutMenuItem = new javax.swing.JMenuItem();

        setTitle("COACH TracingViewer");
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        statusPanel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        eventsLabel.setText("events");
        statusPanel.add(eventsLabel);

        eventsTextField.setBackground(new java.awt.Color(204, 204, 204));
        eventsTextField.setColumns(8);
        eventsTextField.setText(" ");
        statusPanel.add(eventsTextField);

        messagesLabel.setText("messages");
        statusPanel.add(messagesLabel);

        messagesTextField.setBackground(new java.awt.Color(204, 204, 204));
        messagesTextField.setColumns(8);
        statusPanel.add(messagesTextField);

        unmatchedEventsLabel.setText("unmatched events");
        statusPanel.add(unmatchedEventsLabel);

        unmatchedEventsTextField.setBackground(new java.awt.Color(204, 204, 204));
        unmatchedEventsTextField.setColumns(8);
        statusPanel.add(unmatchedEventsTextField);

        getContentPane().add(statusPanel, java.awt.BorderLayout.SOUTH);

        buttonPanel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        clearButton.setText("clear");
        clearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(clearButton);

        uncolorButton.setText("uncolor");
        uncolorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                uncolorButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(uncolorButton);

        trailLabelButton.setText("trail labels");
        trailLabelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                trailLabelButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(trailLabelButton);

        zoomInButton.setText("zoom in");
        zoomInButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomInButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(zoomInButton);

        zoomOutButton.setText("zoom out");
        zoomOutButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomOutButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(zoomOutButton);

        unhideButton.setText("unhide");
        unhideButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                unhideButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(unhideButton);

        identitiesButton.setText("identities");
        identitiesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                identitiesButtonActionPerformed(evt);
            }
        });

        buttonPanel.add(identitiesButton);

        getContentPane().add(buttonPanel, java.awt.BorderLayout.NORTH);

        diagramMainPanel.setLayout(new java.awt.BorderLayout());

        diagramVerticalScrollBar.setMinimumSize(new java.awt.Dimension(17, 400));
        diagramVerticalScrollBar.setPreferredSize(new java.awt.Dimension(17, 400));
        diagramVerticalScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
            public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {
                diagramVerticalScrollBarAdjustmentValueChanged(evt);
            }
        });

        diagramMainPanel.add(diagramVerticalScrollBar, java.awt.BorderLayout.EAST);

        diagramScrollPanel.setVerticalScrollBarPolicy(javax.swing.JScrollPane.VERTICAL_SCROLLBAR_NEVER);
        diagramMainPanel.add(diagramScrollPanel, java.awt.BorderLayout.CENTER);

        getContentPane().add(diagramMainPanel, java.awt.BorderLayout.CENTER);

//        fileMenu.setText("File");
//        connectMenuItem.setText("connect");
//        connectMenuItem.addActionListener(new java.awt.event.ActionListener() {
//            public void actionPerformed(java.awt.event.ActionEvent evt) {
//                connectMenuItemActionPerformed(evt);
//            }
//        });
//        fileMenu.add(connectMenuItem);
//
//        openMenuItem.setText("Open");
//        openMenuItem.addActionListener(new java.awt.event.ActionListener() {
//            public void actionPerformed(java.awt.event.ActionEvent evt) {
//                openMenuItemActionPerformed(evt);
//            }
//        });
//        fileMenu.add(openMenuItem);

//        saveAsMenuItem.setText("Save As ...");
//        saveAsMenuItem.addActionListener(new java.awt.event.ActionListener() {
//            public void actionPerformed(java.awt.event.ActionEvent evt) {
//                saveAsMenuItemActionPerformed(evt);
//            }
//        });
//
//        fileMenu.add(saveAsMenuItem);

        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        optionsMenu.setText("Options");
        threadViewRadioButtonMenuItem.setText("Thread View");
        viewModebuttonGroup.add(threadViewRadioButtonMenuItem);
        threadViewRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                threadViewRadioButtonMenuItemActionPerformed(evt);
            }
        });

        optionsMenu.add(threadViewRadioButtonMenuItem);

        objectViewRadioButtonMenuItem.setSelected(true);
        objectViewRadioButtonMenuItem.setText("Object View");
        objectViewRadioButtonMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                objectViewRadioButtonMenuItemActionPerformed(evt);
            }
        });

        optionsMenu.add(objectViewRadioButtonMenuItem);

        menuBar.add(optionsMenu);

        helpMenu.setText("Help");
        contentsMenuItem.setText("Contents");
        helpMenu.add(contentsMenuItem);

        aboutMenuItem.setText("About");
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setJMenuBar(menuBar);

        pack();
    }//GEN-END:initComponents

//    private void connectMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectMenuItemActionPerformed
//        try {
//            updateTimer.stop();
//            if (currentConnection != null) {
//                dbFacet.close(currentConnection);
//            }
//            diagramPanel.setEventDB(eventDB);
//            IdentityNode.setEventDB(eventDB);
//            ThreadNode.clear();
//            EventRecord.setEventDB(eventDB);
//            int units = (int)(scaleManager.getScale() * Diagram.space_between);
//            int range = getHeight() / units;
//            diagramPanel.setRange(range);
//            eventCount = 0;
//            updateTimer.start();
//        } catch (Exception ex) {
//            javax.swing.JOptionPane.showMessageDialog(null, ex.toString());
//            ex.printStackTrace();
//        }
//    }//GEN-LAST:event_connectMenuItemActionPerformed

//    private void saveAsMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveAsMenuItemActionPerformed
//        try {
//            dbFacet.save("test");
//        } catch (Exception ex) {
//            javax.swing.JOptionPane.showMessageDialog(null, ex.toString());
//        }            
//    }//GEN-LAST:event_saveAsMenuItemActionPerformed

//    private void openMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_openMenuItemActionPerformed
//        try {
//            updateTimer.stop();
//            if (currentConnection != null) {
//                dbFacet.close(currentConnection);
//            }
//            currentConnection = dbFacet.open("test");
//            diagramPanel.setEventDB(currentConnection);
//            IdentityNode.setEventDB(currentConnection);
//            ThreadNode.clear();
//            EventRecord.setEventDB(currentConnection);
//            int units = (int)(scaleManager.getScale() * Diagram.space_between);
//            int range = getHeight() / units;
//            diagramPanel.setRange(range);
//            diagramPanel.updateStatus(currentConnection.getEventCount());
//            diagramPanel.repaintEvents();
//        } catch (Exception ex) {
//            javax.swing.JOptionPane.showMessageDialog(null, ex.toString());
//            ex.printStackTrace();
//        }
//    }//GEN-LAST:event_openMenuItemActionPerformed

    private void objectViewRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_objectViewRadioButtonMenuItemActionPerformed
        diagramPanel.setThreadViewMode(false);
    }//GEN-LAST:event_objectViewRadioButtonMenuItemActionPerformed

    private void threadViewRadioButtonMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_threadViewRadioButtonMenuItemActionPerformed
        diagramPanel.setThreadViewMode(true);
    }//GEN-LAST:event_threadViewRadioButtonMenuItemActionPerformed

    private void identitiesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_identitiesButtonActionPerformed
        IdentityTreeFrame f = new IdentityTreeFrame();
        f.show();
    }//GEN-LAST:event_identitiesButtonActionPerformed

    private void unhideButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_unhideButtonActionPerformed
        IdentityNode.unhideAll();
        ThreadNode.unhideAll();
        diagramPanel.repaintEvents();
    }//GEN-LAST:event_unhideButtonActionPerformed

    private void zoomOutButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomOutButtonActionPerformed
        scaleManager.scaleDown();
        int units = (int)(scaleManager.getScale() * Diagram.space_between);
        int range = getHeight() / units;
        diagramPanel.setRange(range);
        diagramScrollPanel.repaint();        
    }//GEN-LAST:event_zoomOutButtonActionPerformed

    private void zoomInButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomInButtonActionPerformed
        scaleManager.scaleUp();
        int units = (int)(scaleManager.getScale() * Diagram.space_between);
        int range = getHeight() / units;
        diagramPanel.setRange(range);
        diagramScrollPanel.repaint();        
    }//GEN-LAST:event_zoomInButtonActionPerformed

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        int units = (int)(scaleManager.getScale() * Diagram.space_between);
        int range = getHeight() / units;
        diagramPanel.setRange(range);
        diagramScrollPanel.repaint();
    }//GEN-LAST:event_formComponentResized
        
    private void trailLabelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_trailLabelButtonActionPerformed
        String[] labels = diagramPanel.getTrailLabels();
        if (labels.length > 0) {
            TrailLabelData[] trailLabelData = diagramPanel.getTrailLabelData();
            if (trailLabelData != null) {
                if (trailLabelData.length != labels.length) {
                    TrailLabelData[] newData = new TrailLabelData[labels.length];
                    int idx = 0;
                    for(; idx < trailLabelData.length; idx++) {
                        newData[idx] = trailLabelData[idx];
                    }
                    for(; idx < newData.length; idx++) {
                        newData[idx] = new TrailLabelData();
                        newData[idx].name = labels[idx];
                    }
                    trailLabelData = newData;
                }
            } else {                
                trailLabelData = new TrailLabelData[labels.length];
                for (int i = 0; i < trailLabelData.length; i++) {
                    trailLabelData[i] = new TrailLabelData();
                    trailLabelData[i].name = labels[i];
                }
            } 
            TrailLabelDialog d = new TrailLabelDialog(this, trailLabelData);
            trailLabelData = d.getTrailLabelData();
            diagramPanel.setTrailLabelData(trailLabelData);
        }
    }//GEN-LAST:event_trailLabelButtonActionPerformed
    
    private void uncolorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_uncolorButtonActionPerformed
        diagramPanel.uncolour();
        repaint();
    }//GEN-LAST:event_uncolorButtonActionPerformed
    
    private void clearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearButtonActionPerformed
//        diagramPanel.clear();
        repaint();
    }//GEN-LAST:event_clearButtonActionPerformed
        
    private void diagramVerticalScrollBarAdjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {//GEN-FIRST:event_diagramVerticalScrollBarAdjustmentValueChanged
        long start = (long)diagramVerticalScrollBar.getValue();
        if (start != prevStart) {
            prevStart = start;
            diagramPanel.paintEvents(start);
            diagramScrollPanel.repaint();
            repaint();
        }
    }//GEN-LAST:event_diagramVerticalScrollBarAdjustmentValueChanged
    
    private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed

    }//GEN-LAST:event_exitMenuItemActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm

    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new TraceViewerFrame().show();
    }
    
    static public ScaleManager getScaleManager() {
        return scaleManager;
    }

    public void createEventBrowser(EventRecord me, Point p) {
        //open a property browser frame
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int h = 300;
        int w = 450;
        EventBrowser meb = new EventBrowser(me);
        browsers.add(meb);
        meb.setSize(w, h);
        meb.setLocation(((screenSize.width - p.x) > w) ? (p.x + 1) : (p.x - w - 1), (int)((screenSize.height - h) / 2));
        meb.setVisible(true);
    }
    
    public void updateStatusBar(long events, long objects, long messages) {
        eventsTextField.setText("" + events);
        messagesTextField.setText("" + messages);
        unmatchedEventsTextField.setText("" + (events - (2 * messages)));
        diagramVerticalScrollBar.setMaximum((int)events);
        diagramScrollPanel.validate();
        repaint(); 
    }        
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel eventsLabel;
    private javax.swing.JRadioButtonMenuItem objectViewRadioButtonMenuItem;
    private javax.swing.JScrollPane diagramScrollPanel;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JLabel messagesLabel;
    private javax.swing.JTextField eventsTextField;
    private javax.swing.JMenu optionsMenu;
    private javax.swing.JTextField unmatchedEventsTextField;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JMenuItem saveAsMenuItem;
    private javax.swing.JScrollBar diagramVerticalScrollBar;
    private javax.swing.JRadioButtonMenuItem threadViewRadioButtonMenuItem;
    private javax.swing.JButton unhideButton;
    private javax.swing.JPanel diagramMainPanel;
    private javax.swing.JMenuItem connectMenuItem;
    private javax.swing.JButton uncolorButton;
    private javax.swing.JMenuItem openMenuItem;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JButton clearButton;
    private javax.swing.JButton identitiesButton;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton trailLabelButton;
    private javax.swing.JButton zoomInButton;
    private javax.swing.JTextField messagesTextField;
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JMenuItem contentsMenuItem;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JButton zoomOutButton;
    private javax.swing.JLabel unmatchedEventsLabel;
    private javax.swing.ButtonGroup viewModebuttonGroup;
    // End of variables declaration//GEN-END:variables
    
}
