/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.CORBA;

/**
 * This class encapsulates the CORBA::CosTrading trader singleton object.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 *
 * @version 0.1
 */

public abstract class TheTradingService
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     *  To store the CosTrading::Lookup singleton object.
     */
    private static org.omg.CosTrading.Lookup trader_ = null;

    /**
     *  To store the CosTradingRepos::ServiceTypeRepository singleton object.
     */
    private static org.omg.CosTradingRepos.ServiceTypeRepository repos_ = null;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    private
    TheTradingService()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     *  Obtain the CosTrading::Lookup singleton object.
     *
     *  @return The CosTrading::Lookup singleton object.
     */
    public static org.omg.CosTrading.Lookup
    getLookup()
    {
        if(trader_ == null)
        {
            org.omg.CORBA.Object trader_ref = null;
            try {
                trader_ref = TheORB.getORB().resolve_initial_references("TradingService");
            } catch (org.omg.CORBA.ORBPackage.InvalidName ex) {
                // Trading service init ref has not been set.
                return null;
            }            
            setTrader(org.omg.CosTrading.LookupHelper.narrow(trader_ref));
        }
        return trader_;
    }

    /**
     *  Obtain the CosTrading::Register singleton object.
     *
     *  @return The CosTrading::Register singleton object.
     */
    public static org.omg.CosTrading.Register
    getRegister()
    {
        if (getLookup() == null) return null;
        return getLookup().register_if();
    }

    /**
     *  Obtain the CosTrading::Register singleton object.
     *
     *  @return The CosTrading::Register singleton object.
     */
    public static org.omg.CosTradingRepos.ServiceTypeRepository
    getRepository()
    {
        if(repos_ == null)
        {
            getLookup();
        }
        return repos_;
    }

    /**
     *  Set the CORBA::Repository singleton object.
     *
     *  @param repository The CORBA::Repository singleton object.
     */
    public static void
    setTrader(org.omg.CosTrading.Lookup trader)
    {
        trader_ = trader;

        org.omg.CORBA.Object to_narrow = trader_.admin_if().type_repos();
        repos_ = org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.narrow(to_narrow);
    }

}
