/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.Deployment;

import java.net.URL;
import java.util.Hashtable;
import org.omg.Components.Cookie;
import org.omg.Components.Deployment.Assembly;


/**
 * Class AssemblyFactory in charge to provide assembly with a CAD 
 * tree reprensation already load in memory  
 * @author Briclet Frederic
 */
public class AssemblyFactoryImpl extends org.omg.Components.Deployment.AssemblyFactoryPOA 
{
    //hashtable of created assembly
    private    Hashtable assemblies ;
    
    public AssemblyFactoryImpl()
    {    
        assemblies=new java.util.Hashtable();
    }
    
    /**
     * Operation create_assembly return a cookie for retriving later the assembly.
     * if @param is not an valide location an InvalidLocation exception is thrown
     */
    public Cookie 
    create_assembly(String assembly_loc)
    throws org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.CreateFailure
    {
        org.objectweb.ccm.Components.CookieImpl c=
            new org.objectweb.ccm.Components.CookieImpl(""+System.currentTimeMillis());
        URL url=null;
        try{    
           url=new URL(assembly_loc);            
        }
        catch(Exception e){
            e.printStackTrace();
            throw new org.omg.Components.Deployment.InvalidLocation();
        }
        AssemblyImpl ass=new AssemblyImpl(url);
        ass._this(org.objectweb.ccm.CORBA.TheORB.getORB());        
        assemblies.put(c,ass);        
        return c;
    }

    /**
     * Operation lookup return the assembly which match with the @param c cookie given in
     * parameter. If c is not a valide cookie a InvalidAssemblyException is thrown
     */
    public Assembly lookup(org.omg.Components.Cookie c)
        throws org.omg.Components.Deployment.InvalidAssembly
    {
        if(!assemblies.containsKey(c))//The cookie don't match with valid assembly
            throw new org.omg.Components.Deployment.InvalidAssembly();//throw exception
        return  ((AssemblyImpl)assemblies.get(c))._this();         
    }
    
    /** 
     * Operation destroy tear_down the assembly matching with @param c cookie given in 
     * parameter and remove the instance from memory
     */
    public void destroy(org.omg.Components.Cookie c)
    throws org.omg.Components.Deployment.InvalidAssembly, org.omg.Components.RemoveFailure
    {
        if(!assemblies.containsKey(c))
            throw new org.omg.Components.Deployment.InvalidAssembly();
        
         AssemblyImpl ass=(AssemblyImpl)assemblies.remove(c);
         ass.unregisterAssembly();         
         ass.tear_down();      
    }
    
} 
