/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.ccm.descriptor.componentassembly.Connectinterface;
import org.objectweb.ccm.descriptor.componentassembly.beans.ConnectinterfaceBean;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;

/**
 * The connectinterface deployer context defines all the connection requires for
 * connectinterface establishment.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ConnectinterfaceDeployerContext 
       extends ChildDeployerContext
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private ComponentsupportedinterfaceDeployer componentsupportedinterfaceDeployer;
    private ExistinginterfaceDeployer existinginterfaceDeployer;
    private HomeinterfaceDeployer homeinterfaceDeployer;
    private ProvidesportDeployer providesportDeployer;
    private UsesportDeployer usesportDeployer;    
    private ConnectinterfaceBean connectinterface;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessor methods.
    //
    // ==================================================================    
    
    /**
     * @return
     */
    public ComponentsupportedinterfaceDeployer 
    getComponentsupportedinterfaceDeployer()
    {
        return componentsupportedinterfaceDeployer;
    }

    /**
     * @return
     */
    public ExistinginterfaceDeployer 
    getExistinginterfaceDeployer() 
    {
        return existinginterfaceDeployer;
    }

    /**
     * @return
     */
    public HomeinterfaceDeployer 
    getHomeinterfaceDeployer() 
    {
        return homeinterfaceDeployer;
    }

    /**
     * @return
     */
    public ProvidesportDeployer
     getProvidesportDeployer() 
     {
        return providesportDeployer;
    }

    /**
     * @return
     */
    public UsesportDeployer 
    getUsesportDeployer() 
    {
        return usesportDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectComponentsupportedinterfaceDeployer
    (ComponentsupportedinterfaceDeployer deployer) 
    {
        componentsupportedinterfaceDeployer = deployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectExistinginterfaceDeployer(ExistinginterfaceDeployer deployer) 
    {
        existinginterfaceDeployer = deployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectHomeinterfaceDeployer(HomeinterfaceDeployer deployer) 
    {
        homeinterfaceDeployer = deployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectProvidesportDeployer(ProvidesportDeployer deployer) 
    {
        providesportDeployer = deployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectUsesportDeployer(UsesportDeployer deployer)
     {
        usesportDeployer = deployer;
    }
    /**
     * @return
     */
    protected Connectinterface getConnectinterface() {
        return connectinterface;
    }

    /**
     * @param connectinterface
     */
    public void connectConnectinterface(ConnectinterfaceBean connectinterface) {
        this.connectinterface = connectinterface;
        connectDeployerDescription(connectinterface);
    }
        
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError
    {
        if(getConnectinterface()==null)
            getErrorManager()
          .submitException(new InitializationError(this,"Connection on connectinterface bean is required"+
                                               "check the handler code it must specify this connection"));
        if(getUsesportDeployer()==null)
            getErrorManager()
           .submitException(new InitializationError(this,"The useport child must be specified in"+
                                               "connectinterface element check you connectinterface element"+
                                               "description.\n"+getStringifiedDeployerDescription()));
                                               
        if(getProvidesportDeployer()==null&&
           getComponentsupportedinterfaceDeployer()==null&&
           getExistinginterfaceDeployer()==null&&
           getHomeinterfaceDeployer()==null)
            getErrorManager()
           .submitException(new InitializationError(this,"One of The providesport/componentsupportedinterface"+
                                              "existinginterface/homeinterface child must be specified in"+
                                              "connectinterface element check you connectinterface element"+
                                              "description.\n"+getStringifiedDeployerDescription()));
        getLifeCycleManager().stepEndedSuccessFully();
    }
    
}
