/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import org.objectweb.ccm.descriptor.componentassembly.beans.ComponentpropertiesBean;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.util.FileinarchiveDeployer;

/**
 * The componentpropertiesdeployer context defines all the connection requires for
 * setting a component.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ComponentpropertiesDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private FileinarchiveDeployer fileinarchiveDeployer;    
    private ComponentpropertiesBean componentproperties;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================   
    /**
     * @return
     */
    public ComponentpropertiesBean getComponentproperties() {
        return componentproperties;
    }

    /**
     * @return
     */
    public FileinarchiveDeployer getFileinarchiveDeployer() {
        return fileinarchiveDeployer;
    }

    /**
     * @param bean
     */
    public void connectComponentproperties(ComponentpropertiesBean bean) {
        componentproperties = bean;
        connectDeployerDescription(bean);
    }

    /**
     * @param deployer
     */
    public void connectFileinarchiveDeployer(FileinarchiveDeployer deployer) {
        fileinarchiveDeployer = deployer;
    }

    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */  
    public void 
    initialize()
    throws InitializationError
    {
        // TODO Auto-generated method stub
        if(getComponentproperties()==null)       
            getErrorManager()
            .submitException(new InitializationError(this,"Connection on componentproperties is required"+
                                                    "check the handler code it must specify this connection"));
        if(getFileinarchiveDeployer()==null)
            getErrorManager()
            .submitException(new InitializationError(this,"The fileinarchive child"+
                                                     "must be specified in componentproperties element,"+
                                                     "check you element description.\n"+
                                                     getStringifiedDeployerDescription()));
        getLifeCycleManager().stepEndedSuccessFully(); 
    }



}
