/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.Deployer;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.omg.Components.CCMHome;
import org.omg.Components.Deployment.ComponentInstallation;

/**
 * This DestinationDeployer interface defines all the methods that a DestinationDeployer
 * must implement to be used in the deployment machine such as the deployer in charge 
 * of a specific destination kind.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public interface DestinationDeployer extends Deployer{
    
    /**
     * Make the target destination deployer process all the dependencies
     * of the implementation.
     * 
     * @throws FatalDeploymentException if a problem occured
     *  during processing of dependencies
     */
    public void
    resolveDependency()
    throws FatalDeploymentException;
    
    /**
     * InstallHome install the home on the precised destination according
     * to the host requirements, such as dependencies and event factory.
     * 
     * @return The home installed
     * @throws FatalDeploymentException
     */
    public CCMHome
    installHome()
    throws FatalDeploymentException;
     
    /**
     * Return the component installation of the target host used
     * to install the home.
     * 
     * @return The component installation of the target host
     * @throws FatalDeploymentException if a fatal problem occured during installation
     */
    public ComponentInstallation 
    getComponentInstallation()
    throws FatalDeploymentException; 

    /**
     * Remove a previously installed CCMHome
     * 
     * @param home the home to remove
     * @throws FatalDeploymentException if a fatal problem occured during uninstallation
     */
    public void 
    removeHome(CCMHome home)
    throws FatalDeploymentException;
    
}
