/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): Philippe Merle.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import org.ist.coach.DCI.NodeManager;
import org.ist.coach.DCI.NodeManagerHelper;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.objectweb.ccm.descriptor.softpkg.ccm.CodeInstallationFailureException;
import org.objectweb.ccm.descriptor.softpkg.ccm.DependencyResolutionFailureException;
import org.omg.Components.CCMHome;
import org.omg.Components.ConfigValue;
import org.omg.Components.Deployment.ComponentInstallation;
import org.omg.Components.Deployment.ComponentServer;
import org.omg.Components.Deployment.Container;
import org.omg.Components.Deployment.InstallationFailure;
import org.omg.Components.Deployment.InvalidLocation;

/**
 * The NodeDestinationDeployer is the DestinationDeployer able to install
 * a home on a DCI node.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class NodeDestinationDeployer 
       extends DestinationDeployerBase 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    private NodeManager node;

    private ComponentServer cs;

    private Hashtable containerTable;

    private java.util.List config_values;

    private NodeDependencyVisitorDeployer depVisitor;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    public
    NodeDestinationDeployer()
    {
        containerTable = new Hashtable();
        config_values = new java.util.LinkedList();
        depVisitor = new NodeDependencyVisitorDeployer();
    }

    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================

    /**
     * Internal method to install a home
     */
    private CCMHome 
    installCCMHome(
        ConfigValue[] csConfig,
        ConfigValue[] contConfig,
        ConfigValue[] homeConfig,
        String implID,
        String entryPoint)
    throws FatalDeploymentException 
    {
        CCMHome home = null;
        try {
            if (cs == null)
                cs =
                    getNode()
                        .provide_server_activator()
                        .create_component_server(
                        csConfig);
            Container cont = cs.create_container(contConfig);
            home = cont.install_home(implID, entryPoint, homeConfig);
            containerTable.put(home, cont);
        } catch (Exception e) {
            e.printStackTrace();
            throw new FatalDeploymentException(this,e,"Cannot deploy the CCMHome on the required destination");
        }
        return home;
    }

    /**
     * Internal method to retrieve a nodeManager
     * @return
     * @throws Exception
     */
    private NodeManager 
    getNode() 
    throws Exception 
    {
        if (node == null)
        {
            node = getNodeDeployer().getNodeManager();
        }
        return node;
    }

    /**
     * Internal method to start a HTTP server for component archive uploading
     * @param input the input on the file to upload
     * @param name the file name to upload
     * @return The URL to use to download file
     * @throws java.io.IOException if I/O problem occured
     */
    private String 
    startHttpServer(InputStream input, String name)
    throws java.io.IOException 
    {
        (new Thread(new org
            .objectweb
            .ccm
            .deploytool
            .MicroServerHttp(
                getRootDeployerContext().getServerSocket(),
                input,
                name)))
            .start();
        return "http://"
            + java.net.InetAddress.getLocalHost().getHostAddress()
            + ':'
            + getRootDeployerContext().getServerSocket().getLocalPort()
            + '/'
            + name;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ================================================================== 

    /**
     * Make the target destination deployer process all the dependencies
     * of the implementation.
     * 
     * @throws FatalDeploymentException if a problem occured
     *  during processing of dependencies
     */
    public void 
    resolveDependency() 
    throws FatalDeploymentException 
    {
        try {
            /*config_values.addAll(Arrays.asList(getSoftpkgDeployer()
                                                .processDependencies
                                                    (getComponentInstallation())));*/
            getSoftpkgDeployer().visitDependencyDeployers(depVisitor);
            /* List l=Arrays.asList(getImplementationDeployer()
                                  .install(getComponentInstallation()));
               
              config_values.addAll(l);    */
            getImplementationDeployer().visitDependencyDeployers(depVisitor);
            getImplementationDeployer().visitCodeDeployers(depVisitor);

            try {
                String url =
                    startHttpServer(
                        getSoftpkgDeployer().getComponentArchiveStream(),
                        getSoftpkgDeployer().getSoftpkg().getName() + ".car");
                getComponentInstallation().install(
                    getSoftpkgDeployer().getSoftpkg().getName(),
                    url);

            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            } catch (InvalidLocation e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            } catch (InstallationFailure e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
                throw new FatalDeploymentException(
                    e,
                    "Cannot install implementation in target node");
            }

        } catch (DependencyResolutionFailureException e) {
            getErrorManager().submitException(e);
        } catch (CodeInstallationFailureException e) {
            e.printStackTrace();
            getErrorManager().submitException(e);
        }
    }

    /**
     * InstallHome install the home on the precised destination according
     * to the host requirements, such as dependencies and event factory.
     * 
     * @return The home installed
     * @throws FatalDeploymentException
     */
    public CCMHome 
    installHome() 
    throws FatalDeploymentException 
    {

        ConfigValue[] config_home =
        /*(ConfigValue[])
                                         config_values.toArray(new ConfigValue[config_values.size()]);*/
        depVisitor.getConfigValue();

        return installCCMHome(
            new ConfigValue[0],
            new ConfigValue[0],
            config_home,
            getImplementationDeployer().getImplementation().getId(),
            depVisitor.getEntryPoint());
    }

    /**
     * Return the component installation of the target host used
     * to install the home.
     * 
     * @return The component installation of the target host
     * @throws FatalDeploymentException if a fatal problem occured during installation
     */
    public ComponentInstallation 
    getComponentInstallation()
    throws FatalDeploymentException 
    {
        try {

            return getNode().provide_component_installation();
        } catch (Exception e) {
            getLogger().log("Exception occured during retrieving of Node");
        throw new FatalDeploymentException(this,e,"Cannot access to the componentinstallation on the required destination");
        }
    }

    /**
     * Remove a previously installed CCMHome
     * 
     * @param home the home to remove
     * @throws FatalDeploymentException if a fatal problem occured during uninstallation
     */
    public void 
    removeHome(CCMHome home) 
    throws FatalDeploymentException 
    {
        try {
            Container cont = (Container) containerTable.remove(home);
            if(cont==null)
                return ;
            cont.remove_home(home);
            
            if(cont.get_homes().length==0){
                if(cont.get_component_server().get_containers().length==1)
                    cont.get_component_server().remove();
                else 
                    cont.remove();                            
            } 
          
        } catch (Exception e) {
            e.printStackTrace();
            throw new FatalDeploymentException(e);
        }
    }

}
