/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.installer;


import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.*;

/**
 * Process the partitionning deployment.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class PartitioningDeployer extends PartitioningDeployerContext
{
    
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The partition deploy method is in charge to deploy all the collocation
     * ,processcollocation and homeplacement. 
     *
     * @throws FatalDeploymentException if a fatal problem occured
     */
    public void 
    deploy()
    throws FatalDeploymentException
    {
       
        //deploy homeplacement
        traverse(this,getHomeplacementDeployers());
        //deploy componentplacement
        traverse(this,getComponentplacementDeployers());
        //deploy hostcollocation
        traverse(this,getHostcollocationDeployers());
        // deployment of the processlocation list
        traverse(this,getProcesscollocationDeployers());        

        getLifeCycleManager().stepEndedSuccessFully();        
    }
    
    /**
     * Remove all the installed home and component precised by the partioning
     * element.
     *
     * @throws FatalDeploymentException if a fatal problem occured    
     */    
    public void 
    tearDown()
    throws FatalDeploymentException
    {
        traverse(this,getComponentplacementDeployers()); 
        traverse(this,getHomeplacementDeployers());    
        traverse(this,getHostcollocationDeployers());
        traverse(this,getProcesscollocationDeployers());  
        getLifeCycleManager().stepEndedSuccessFully();  
    }
    
    /**
     * HomeplacementDeployer visitor method to install a homeplacement.
     * @param homeplacementDeployer the homeplacementDeployer to execute
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */
    public void 
    visit(HomeplacementDeployer homeplacementDeployer, DeployerInactiveState status)
    throws FatalDeploymentException{
        homeplacementDeployer.deploy();
    }
    /**
     * HomeplacementDeployer visitor method to uninstall a homeplacement.
     * @param homeplacementDeployer the homeplacementDeployer to uninstall
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */   
    public void 
    visit(HomeplacementDeployer homeplacementDeployer, DeployerActiveState status)
    throws FatalDeploymentException{
        homeplacementDeployer.tearDown();
    }
    /**
     * componentplacementDeployer visitor method to execute a componentplacement.
     * @param componentplacementDeployer the homeplacementDeployer to execute
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */   
    public void 
    visit(ComponentplacementDeployer componentplacementDeployer, DeployerInactiveState status)
    throws FatalDeploymentException{
        componentplacementDeployer.deploy();
    }
    /**
     * componentplacementDeployer visitor method to uninstall a componentplacement.
     * @param componentplacementDeployer the homeplacementDeployer to uninstall
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */   
    public void 
    visit(ComponentplacementDeployer componentplacementDeployer, DeployerActiveState status)
    throws Exception{
        componentplacementDeployer.tearDown();
    }
    /**
     * hostcollocationDeployer visitor method to execute a hostcollocation.
     * @param hostcollocationDeployer the hostcollocationDeployer to execute
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */   
    public void 
    visit(HostcollocationDeployer hostcollocationDeployer, DeployerInactiveState status)
    throws Exception{
       hostcollocationDeployer.deploy();
    }
    /**
     * hostcollocationDeployer visitor method to uninstall a hostcollocation.
     * @param hostcollocationDeployer the hostcollocationDeployer to uninstall
     * @param status the partitioning current status
     * @throws FatalDeploymentException if a fatal problem occured 
     */   
    public void 
    visit(HostcollocationDeployer hostcollocationDeployer, DeployerActiveState status)
    throws Exception{
        hostcollocationDeployer.tearDown();
    }
    
    
}

