/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.managers;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.DeploymentLogger;

/**
 * The default Deployment logger implementation, which log the event message into
 * a file.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DefaultDeploymentLogger 
       implements DeploymentLogger 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private PrintWriter print;
    private File file;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    /**
     * The onstructor precising the text file to
     * write in
     */
    public DefaultDeploymentLogger(String filepath)
    throws Exception
    {
            File file=new File(filepath);
            file.createNewFile();
            print= new PrintWriter(new FileWriter(file));
    }
    /**
     * The default constructor
     */
    public DefaultDeploymentLogger()
    {
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    private void
    setTemporaryLogFile()
    {
    	try{
			file=File.createTempFile("DeploymentLog"+System.currentTimeMillis(),".log");
        	file.createNewFile();
        	print= new PrintWriter(new FileWriter(file));
    	}
    	catch(Exception e){
    		e.printStackTrace();
    	}

    }
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    public void
    setLogFile(String filepath)
    throws Exception
    {
        file=new File(filepath);
        file.createNewFile();
        print= new PrintWriter(new FileWriter(file));
    }
    
    
    public void
    removeLogFile()
    {
        try{
            print.close();
            file.delete();
        }
        catch(Exception e)
        {
            System.err.println("A problem occured during log file removing");
        }
    }
    /**
     * Log the given mesage @param ch in the current open file
     * @param ch the message to log 
     */
    public void 
    log(String ch) 
    {
        if(print==null)
            setTemporaryLogFile();
        else
        {
            print.println(ch);
            print.flush();
        }
    }
}
