/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Christophe Contreras.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.root;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;
import org.omg.Components.CCMObject;
import org.omg.Components.InvalidConfiguration;

/**
 * The ComponnetassemblyDeployer is the root of all the deployer tree. 
 * @author briclet
 *
 * 
 */
public class ComponentassemblyDeployer
    extends ComponentassemblyDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Deploy start the execution of the deployment process.
     * 
     * @throws FatalDeploymentException if a fatal error occured
     */
    public void deploy()
    throws FatalDeploymentException
    {
        getDeploymentLogger().log("<<STEP2>> Begin the information reading");
        // Load the imbricated CSD and CAD, using the extension manager tool
        //if (getComponentfilesDeployer() != null)
        //    getComponentfilesDeployer().deployPartial(deployTable);
        getDeploymentLogger().log("<</STEP2>> Information reading ended");
        // Home deployment and instantiation of components
        getDeploymentLogger().log("<<STEP3>> Begin the component installation");
        getPartitioningDeployer().deploy();
        getDeploymentLogger().log("<</STEP3>>Component installation ended");
        getDeploymentLogger().log("<<STEP4>> Establishing component connections");
        if (getConnectionsDeployer() != null)
            getConnectionsDeployer().deploy();
           
        getLifeCycleManager().stepEndedSuccessFully();
        getDeploymentLogger().log("<</STEP4>> Component connections established" );
    }
    /**
     * Configure complete all the deployed components.
     * 
     * @throws FatalDeploymentException if a fatal error occured.
     */
    public void configueCompleteComponents()
    throws FatalDeploymentException
    {
        CCMObject [] list=getComponents();
        try{
            for(int i=0;i<list.length;i++){
               (new ComponentCompleter(list[i])).start();
            }
        }
        catch(StartingError e){
            throw new FatalDeploymentException(e.getException(),"A component cannot be started\n");
        }

    }
    /**
     * Tear down the built assembly.
     * 
     * @throws FatalDeploymentException if a fatal error occured.      
     */
    public void tearDown()
    throws FatalDeploymentException
    {
        getDeploymentLogger().log("<</STEP5>> Teardowning the assembly" );
        //Break connections
        if (getConnectionsDeployer() != null)
            getConnectionsDeployer().tearDown();
        getPartitioningDeployer().tearDown();        

        getLifeCycleManager().stepEndedSuccessFully();
        getDeploymentLogger().log("<</STEP5>> Assembly teardowned successfully" );
    }
    


}
