/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s):

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.gui;

import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

/**
 * This is an extension to the Apollon generated framework around componentassembly DTD
 *  it looks for subelements that could contain files to archive
 *  and calls their own zipper extensions to do the same job
 *  any found file name is sent to a zip factory
 *
 */
public class ComponentassemblyZipper
{
    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    /**
     *  The zipCall method used to go through the element children
     *
     * @param ComponentassemblyBean componentassembly the element to parse
     * @param ZipFactory zip_factory the factory to send file names
     */
    public void
    zipCall(
        ComponentassemblyBean componentassembly
        , org.objectweb.openccm.deployment.gui.ZipFactory zip_factory
        )
    {
        // components files
        ComponentfilesBean componentfiles
         = (ComponentfilesBean) componentassembly.getComponentfiles();
        // goes through sub element and call its zip method
        if (componentfiles != null)
        {
            ComponentfilesZipper sub_zipper
             = (ComponentfilesZipper) componentfiles
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(componentfiles, zip_factory);
        }

        // partitioning
        PartitioningBean partitioning
         = (PartitioningBean) componentassembly.getPartitioning();
        // goes through sub element and call its zip method
        if (partitioning != null)
        {
            PartitioningZipper sub_zipper
             = (PartitioningZipper) partitioning
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(partitioning, zip_factory);
        }
    }
}
