/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.softpkg.ccm;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.softpkg.beans.DependencyBean;

/**
 * The dependency deployer context defines all the connection requires for
 * resolve a dependency.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class DependencyDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private DependencyBean dependency;
    private FileinarchiveDeployer fileinarchiveDeployer;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================    
    /**
     * @return
     */
    public DependencyBean 
    getDependency() 
    {
        return dependency;
    }

    /**
     * @param bean
     */
    public void 
    connectDependency(DependencyBean bean) 
    {
        dependency = bean;
        connectDeployerDescription(bean);
    }

    /**
     * @return
     */
    public FileinarchiveDeployer 
    getFileinarchiveDeployer() 
    {
        return fileinarchiveDeployer;
    }

    /**
     * @param deployer
     */
    public void
    connectFileinarchiveDeployer(FileinarchiveDeployer deployer) 
    {
        fileinarchiveDeployer = deployer;
    }
    
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError 
    {
        // TODO Auto-generated method stub
        if(getDependency()==null)
            getErrorManager()
            .submitException(new InitializationError(this,"Connection on dependency bean is required"+
                                                    "check the handler code it must specify this connection"));
        getLifeCycleManager().stepEndedSuccessFully();
    }



}
