/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.ccm.descriptor.softpkg.ccm;

import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.HandlerBase;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBeanImpl;

/**
 * This handler is used to produce softpkgdeployer from an
 * softpkg bean .
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class SoftpkgDeployerHandler 
    extends HandlerBase 
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private SoftpkgDeployer _deploy;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================
    /**
     * Produce the softpkg deployer binded
     * to the given @param cons.
     * 
     * @param cons The softpkg bean 
     * @return the deployer produced in initialize state
     * @throws InitializationError thrown if a problem is detected
     */ 
    public Object 
    getInstance(SoftpkgBeanImpl cons)
    throws InitializationError 
    {
        if (_deploy == null) {
            _deploy = new SoftpkgDeployer();
            _deploy.connectSoftpkg(cons);
            applyCommonConfig(_deploy);            
            if (cons.getImplementationList() != null)
                for (java.util.Iterator i =
                    cons.getImplementationList().iterator();
                    i.hasNext();
                    ) {
                    ImplementationDeployer implDeployer =
                        (ImplementationDeployer) getDeployer(i.next());

                    _deploy.connectImplementationDeployer(implDeployer);
                }

            if (cons.getDependencyList() != null)
                for (java.util.Iterator i = cons.getDependencyList().iterator();
                    i.hasNext();
                    ) {
                    DependencyDeployer dd =
                        (DependencyDeployer) getDeployer(i.next());

                    _deploy.connectDependencyDeployer(dd);

                }
            if (cons.getDescriptorList() != null)
                for (java.util.Iterator i = cons.getDescriptorList().iterator();
                    i.hasNext();
                    )
                    _deploy.connectDescriptorDeployer(
                        (DescriptorDeployer) getDeployer(i.next()));

            _deploy.initialize();
        }
        return (_deploy);
    }
    
}
