/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Tran Huynh (tran.huynh@fr.thalesgroup.com)
Contributor(s): .

====================================================================*/

package org.objectweb.ccm.scripts;

/**
 * This class manages the OpenCCM XMI Compiler.
 */
public class XMICompiler {

    /**
     * The options manager.
     */
    private IR3XMIOptionsManager manager_;

    /**
     * Version.
     */
    private java.lang.String version_;

    /**
     * Constructor.
     *
     * @param manager The options manager.
     * @param version The version.
     */
    public 
    XMICompiler(IR3XMIOptionsManager manager,
                java.lang.String version) {
        manager_ = manager;
        version_ = version;
    }

    /**
     * This method prints a message to the standard output.
     */
    protected void 
    message(String text) {
        System.out.println(version_ + " : " + text);
    }

    /**
     * This method analyzes the command line arguments.
     *
     * @return True if everything is ok, false otherwise.
     */
    public boolean run(java.lang.String[] cmdline) {
        if (manager_.analyse(cmdline) == -1) {
            return false;
        }
        return true;
    }

    /**
     * This method returns the options manager.
     *
     * @return The options manager.
     */
    public IR3XMIOptionsManager 
    getManager() {
        return manager_;
    }
}
