/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cidl.api;

// Package dependencies.
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.ComponentDecl;
import org.objectweb.corba.ast.api.HomeDecl;
import org.objectweb.corba.ast.api.InterfacePortDecl;
import org.objectweb.corba.ast.api.EventPortDecl;
import org.objectweb.corba.ast.api.Scope;
import org.objectweb.corba.ast.api.TypeRef;
import java.util.Set;


/**
 * This interface allows you to get classes dependencies for an IDL3 declaration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface IDL3DeclarationDependencies
         extends IDL2DeclarationDependencies
{
    /**
     * Get Home File dependencies.
     * This includes all inherited Homes.
     *
     * @param home - The home declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getHomeDependencies(HomeDecl home,
                        Set visited);

    /**
     * Get Component File dependencies.
     * This includes all inherited Components.
     *
     * @param comp - The Component declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getComponentDependencies(ComponentDecl comp,
                             Set visited);

    /**
     * Get Interface Port Declaration File dependencies.
     * This includes all inherited interfaces.
     *
     * @param itf_port - The interface port declaration.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getInterfacePortDependencies(InterfacePortDecl itf_port,
                                 boolean server,
                                 Set visited);

    /**
     * Get Event port Declaration File dependencies.
     * This includes all inherited interfaces.
     *
     * @param event_port - The event port declaration.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getEventPortDependencies(EventPortDecl event_port,
                             boolean server,
                             Set visited);

    /**
     * Get dependencies for an IDL3 declaration.
     *
     * @param decl - Get dependencies for this declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getDeclarationDependencies(Declaration decl,
                               Set visited);

    /**
     * Get Type Reference dependencies class files.
     *
     * @param aconst - Get dependencies from this type reference.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefDependencies(TypeRef type,
                           Set visited);
}
