/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.java.ast.lib;

import org.objectweb.corba.generator.java.ast.api.StaticObject;
import org.objectweb.corba.generator.java.ast.api.AttributeObject;

/**
 * This is the representation for java Attributes.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class AttributeObjectImpl
     extends ObjectBaseImpl
  implements AttributeObject
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The delegation for a static */
    private StaticObject static_delegate_;

    /** The Attribute type */
    private String type_;

    /** The Attribute initial value */
    private String initial_value_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    AttributeObjectImpl()
    {
        // Init internal states
        super();
        static_delegate_ = new StaticObjectImpl();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.corba.generator.java.ast.api.StaticObject
    //
    // ==================================================================

    /**
     * Set static this object.
     *
     * @param b - The value to set.
     */
    public void
    setStatic(boolean b)
    {
        static_delegate_.setStatic(b);
    }

    /**
     * Is this object static ?
     *
     * @return True if it is static, else false.
     */
    public boolean
    isStatic()
    {
        return static_delegate_.isStatic();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.corba.generator.java.ast.api.AttributeObject
    //
    // ==================================================================

    /**
     * Set its type.
     *
     * @param type - The type to set.
     */
    public void
    setType(String type)
    {
        type_ = type;
    }

    /**
     * Obtain its type.
     *
     * @return Its type.
     */
    public String
    getType()
    {
        return type_;
    }

    /**
     * Set its initial value.
     *
     * @param value - The value to set.
     */
    public void
    setInitialValue(String value)
    {
        initial_value_ = value;
    }

    /**
     * Obtain its initial value.
     *
     * @return Its initial value.
     */
    public String
    getInitialValue()
    {
        return initial_value_;
    }
}
