/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.StorageHomeDecl;
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeDecl;
import org.objectweb.corba.ast.api.StorageHomeKeyDecl;
import org.objectweb.corba.ast.api.StorageHomeFactoryDecl;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.Declaration;

/** To access Java AST. */
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import java.util.Vector;
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates Storage Home Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class StorageHomeMapping
  implements org.objectweb.corba.generator.psdl.api.JavaMappingObject
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Storage Home declaration. */
    private StorageHomeDecl sh_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param sh - The Storage Home to map.
     */

    public StorageHomeMapping(StorageHomeDecl sh)
    {
        // Init internal state
        sh_ = sh;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Has this abstract storage home an operation declaration ?
     * (including implemented abstract storage homes)
     *
     * @param ash - The abstract storage home to test.
     *
     * @return True if an operation is contained, else false.
     **/
    public boolean
    hasOperation(AbstractStorageHomeDecl ash)
    {
        Declaration[] decls = null;
        decls = ash.getContents(true, DeclarationKind.dk_psdl_operation);
        if (decls.length > 0)
        {
            return true;
        }
        else
        {
            // Check implemented abstract storage types
            AbstractStorageHomeDecl[] implemented = null;

            implemented = ash.getAbstractStorageHomeList().getAbstractStorageHomes();
            for (int i=0; i<implemented.length; i++)
            {
                if ( hasOperation(implemented[i]) )
                    return true;
            }
            return false;
        }
    }

    /**
     * Map Storage Home create Operations into Java methods.
     *
     * @param generator - The PSDL to Java generator.
     * @param obj - The StorageHome mapping object.
     */
    private void
    mapStorageHomeCreateOperations(
        org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator,
        ClassObject obj)
    throws GenerationException
    {
        MethodObject method = null;
        ParameterObject param = null;
        StorageTypeDecl managed = null,
                        managed_base = null;
        AbstractStorageTypeDecl[] ast_impl_directly = null;;
        StorageTypeStateMemberDecl[] all_states = null;
        Vector vect = new Vector();
        java.util.List states = null;


        managed = sh_.getStorageType();

        /* Get all storagetype's state members */
        // get managed storagetype state members
        managed_base = managed.getBaseStorageType();
        if (managed_base != null)
        {
            vect.addAll( generator.getDeclarations(managed_base, DeclarationKind.dk_storage_type_state_member) );
            all_states = (StorageTypeStateMemberDecl[])vect.toArray(new StorageTypeStateMemberDecl[0]);
        }
        // get state members implemented directly
        ast_impl_directly = managed.getAllImplementedAbstractStorageTypes().getAbstractStorageTypes();
        for (int j=0; j<ast_impl_directly.length; j++)
        {
            vect.addAll(
                generator.getDeclarations(ast_impl_directly[j], DeclarationKind.dk_storage_type_state_member) );
        }
        // get own state members
        vect.addAll( generator.getDeclarations(managed, DeclarationKind.dk_storage_type_state_member) );
        all_states = (StorageTypeStateMemberDecl[])vect.toArray(new StorageTypeStateMemberDecl[0]);

        /* The generic create method (not in api) */
        method = new MethodObjectImpl();
        method.addComment("The generic create method.");
        method.setName("create_st_incarnation");
        method.setReturnType( generator.getTranslator().getAbsoluteName(managed) );
        method.getImpl().setMacro("SH_CREATE_ST_INCARNATION");
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(managed));
        obj.addMethod(method);

        /* Create method that takes a parameter for each of its storagetype's
           state members and returns an incarnation */
        method = new MethodObjectImpl();
        method.addComment("A create method.");
        method.setName("_create");
        method.setReturnType( generator.getTranslator().getAbsoluteName(managed) );
        states = new java.util.ArrayList();
        for (int i=0; i<all_states.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.StateMemberMapping state
                = new StateMemberMapping(all_states[i]);

            state.toJavaParameter(generator.getTranslator(), method);
            states.add(all_states[i]);
        }
        method.getImpl().setMacro("SH_CREATE1_METHOD");
        method.getImpl().addContextValue("states", states);
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(managed));
        obj.addMethod(method);

        /* Create method that takes a parameter for each of its storagetype's
           reference representation member. */
        method = new MethodObjectImpl();
        method.addComment("A create method.");
        method.setName("_create");
        method.setReturnType( generator.getTranslator().getAbsoluteName(managed) );
        states = new java.util.ArrayList();
        for (int i=0; i<all_states.length; i++)
        {
            if (all_states[i].getType().isRef())
            {
                org.objectweb.corba.generator.psdl.api.StateMemberMapping state
                    = new StateMemberMapping(all_states[i]);

                state.toJavaParameter(generator.getTranslator(), method);
                states.add(all_states[i]);
            }
        }
        method.getImpl().setMacro("SH_CREATE1_METHOD");
        method.getImpl().addContextValue("states", states);
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(managed));
        obj.addMethod(method);

        /* Create method that takes a parameter for each of its storagetype's
           state members plus a CosPersistentState.YielRef parameter
           and returns a reference. */
        method = new MethodObjectImpl();
        method.addComment("A create method.");
        method.addComment(" ");
        method.setName("_create");
        method.setReturnType("byte[]");
        states = new java.util.ArrayList();
        for (int i=0; i<all_states.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.StateMemberMapping state
                = new StateMemberMapping(all_states[i]);

            state.toJavaParameter(generator.getTranslator(), method);
            states.add(all_states[i]);
        }
        param = new ParameterObjectImpl();
        param.setName("yr");
        param.setType("org.omg.CosPersistentState.YieldRef");
        method.addParameter(param);
        method.getImpl().setMacro("SH_CREATE2_METHOD");
        method.getImpl().addContextValue("states", states);
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(managed));
        obj.addMethod(method);

        /* Create method that takes a parameter for each of its storagetype's
           state members plus a CosPersistentState.YielRef parameter
           reference representation member. */
        method = new MethodObjectImpl();
        method.addComment("A create method.");
        method.setName("_create");
        method.setReturnType("byte[]");
        states = new java.util.ArrayList();
        for (int i=0; i<all_states.length; i++)
        {
            if (all_states[i].getType().isRef())
            {
                org.objectweb.corba.generator.psdl.api.StateMemberMapping state
                    = new StateMemberMapping(all_states[i]);

                state.toJavaParameter(generator.getTranslator(), method);
                states.add(all_states[i]);
            }
        }
        param = new ParameterObjectImpl();
        param.setName("yr");
        param.setType("org.omg.CosPersistentState.YieldRef");
        method.addParameter(param);
        method.getImpl().setMacro("SH_CREATE2_METHOD");
        method.getImpl().addContextValue("states", states);
        method.getImpl().addContextValue("st_type", generator.getTranslator().getAbsoluteName(managed));
        obj.addMethod(method);
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Map a Storage Home definition into a java Class.
     *
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava(org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator generator)
    throws GenerationException
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        MethodObject method = null;
        AttributeObject att = null;
        ParameterObject param = null;
        AbstractStorageHomeDecl[] implemented = null,
                                  impl_directly = null;;
        StorageHomeDecl base = null;
        StorageHomeKeyDecl[] keys = null;
        StorageHomeFactoryDecl[] factories = null;
        Vector vect = null;

        clazz = new ClassObjectImpl(sh_.getName());
        clazz.addComment(" Storage Home "+sh_.getName());
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM PSDL Compiler");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage(generator.getTranslator().getPackage(sh_));

        // Add Implemented Abstract Storage Home by sh
        implemented = sh_.getAbstractStorageHomeList().getAbstractStorageHomes();
        for (int i=0; i<implemented.length; i++)
        {
            clazz.addImplementedObject(generator.getTranslator().getAbsoluteName(implemented[i]));
        }

        // Add Implemented storage home base interface
        clazz.addImplementedObject("org.objectweb.openccm.services.pss.common.api.StorageHomeBase");

        // Check if any of the abstract storagehome implemented declares an operation
        int i = 0;
        boolean found = false;
        clazz.setAbstract(false);
        while ((i<implemented.length) && !found)
        {
            if (hasOperation(implemented[i]))
            {
                clazz.setAbstract(true);
                found = true;
            }
            i++;
        }

        // Add Inherited Storage Home if there is one
        base = sh_.getBaseStorageHome();
        if (base != null)
            clazz.addInheritedObject(generator.getTranslator().getAbsoluteName(base));
        else
            clazz.addInheritedObject("org.objectweb.openccm.services.pss.jdo.lib.StorageHomeBase");

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        clazz.addConstructor(ct);

        // Map Finder operations (i.e. key mapping) and factories
        // of directly implemented Abstract storage home
        impl_directly = sh_.getDirectlyImplementedAbstractStorageHomes();
        for (int j=0; j<impl_directly.length; j++)
        {
            // Map factories
            vect = generator.getDeclarations(impl_directly[j], DeclarationKind.dk_storage_home_factory);
            factories = (StorageHomeFactoryDecl[])vect.toArray(new StorageHomeFactoryDecl[0]);
            for (int k=0; k<factories.length; k++)
            {
                org.objectweb.corba.generator.psdl.api.PSDLFactoryMapping factory
                    = new PSDLFactoryMapping(factories[k]);
                factory.toJava(generator, clazz, impl_directly[j].getAbstractStorageType(), sh_.getStorageType());
            }
            // Map keys
            vect = generator.getDeclarations(impl_directly[j], DeclarationKind.dk_storage_home_key);
            keys = (StorageHomeKeyDecl[])vect.toArray(new StorageHomeKeyDecl[0]);
            for (int k=0; k<keys.length; k++)
            {
                org.objectweb.corba.generator.psdl.api.PSDLKeyMapping key
                    = new PSDLKeyMapping(keys[k]);
                key.toJava(generator, clazz, impl_directly[j].getAbstractStorageType());
            }
        }

        // Map create operations
        mapStorageHomeCreateOperations(generator, clazz);

        // Add the get_rid method
        method = new MethodObjectImpl();
        method.addComment("Get the Repository ID of a storage home.");
        method.setName("get_rid");
        method.setReturnType("java.lang.String");
        method.getImpl().setMacro("SH_GET_RID_METHOD");
        method.getImpl().addContextValue("sh_rid", sh_.getId());
        clazz.addMethod(method);

        // Add the get_rid method
        method = new MethodObjectImpl();
        method.addComment("Get the Repository ID of the managed storage type.");
        method.setName("get_st_rid");
        method.setReturnType("java.lang.String");
        method.getImpl().setMacro("SH_GET_ST_RID_METHOD");
        method.getImpl().addContextValue("st_rid", sh_.getStorageType().getId());
        clazz.addMethod(method);

        generator.getJavaRepository().addObject(clazz);
    }

}
