/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.translator.ast2idl.lib;

/**
 * Utility class to translate PSDL types in string format.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class AST_PSDLTranslator
     extends org.objectweb.corba.generator.translator.ast2idl.lib.AST_IDLTranslator
  implements org.objectweb.corba.generator.translator.ast2idl.api.AST_PSDLTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    AST_PSDLTranslator()
    {
        super();
        keywords_.add("as");
        keywords_.add("factory");
        keywords_.add("implements");
        keywords_.add("key");
        keywords_.add("of");
        keywords_.add("primary");
        keywords_.add("ref");
        keywords_.add("scope");
        keywords_.add("state");
        keywords_.add("storagehome");
        keywords_.add("storagetype");
        keywords_.add("stores");
        keywords_.add("strong");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Translate a psdl type into its idl / psdl format.
     *
     * @param type - The psdl type reference to translate.
     *
     * @return The resulting string.
     **/
    public String
    toPSDL(org.objectweb.corba.ast.api.PsdlTypeRef type)
    {
        org.objectweb.corba.ast.api.TypeRef idl_type = null;
        org.objectweb.corba.ast.api.StorageTypeBase st_base = null;

        if (type == null)
        {
            return "";
        }
        idl_type = type.getTypeRef();
        if (idl_type != null)
        {
            return toIDL(idl_type);
        }
        else
        {   // type is a psdl type
            st_base = type.getStorageTypeBase();
            return getAbsoluteName(st_base);
        }
    }
}
