/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.translator.idl2java.api;

// Package dependencies.
import org.objectweb.corba.generator.java.ast.api.MethodObject;
import org.objectweb.corba.generator.java.ast.api.ClassObject;

/**
 * This interface translates some IDL declarations to their java mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface IDL_JavaTranslator
         extends org.objectweb.corba.generator.translator.idl2java.api.CommonTranslator
{
    /**
     * Map an IDL operation.
     *
     * @param op - The operation to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_operation(org.objectweb.corba.ast.api.OperationDecl op);

    /**
     * Map an IDL attribute to an accessor operation.
     *
     * @param att - The attribute to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_attribute_accessor(org.objectweb.corba.ast.api.AttributeDecl att);

    /**
     * Map an IDL attribute to a mutator operation.
     *
     * @param att - The attribute to map.
     *
     * @return The MethodObject binding to this operation.
     **/
    public MethodObject
    map_attribute_mutator(org.objectweb.corba.ast.api.AttributeDecl att);

    /**
     * Map a parameter list for a method.
     *
     * @param method - Add parameters mapping to this method.
     * @param list - The parameter list to map.
     **/
    public void
    map_parameters(MethodObject method,
                   org.objectweb.corba.ast.api.ParameterList list);

    /**
     * Map an exeception list for a method.
     *
     * @param method - Add exceptions mapping to this method.
     * @param list - The exception list to map.
     **/
    public void
    map_exceptions(MethodObject method,
                   org.objectweb.corba.ast.api.ExceptionList list);

    /**
     * Generate operations mapping for an interface.
     *
     * @param clazz - Add methods to the segment class.
     **/
    public void
    mapInterfaceOps(ClassObject clazz,
                    org.objectweb.corba.ast.api.InterfaceDecl itf);
}
