/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.trace.PI;

/**
 * @author offroy@lifl.fr
 *
 * used to read CORBA::ClientRequestInfo
 *
 */
//TODO change logLevel to a more configurable way to get CORBA information about current call
public class ClientRequestInfoReader extends RequestInfoReader {
	/**
	 * CORBA Client Portable Interceptor Trace level
	 */
	final static private short logLevelClientStart = 2;

	/**
	 * Constructor 
	 * @param logLevel
	 **/
	public ClientRequestInfoReader(short logLevel) {
		super(logLevel);
	}

	/************************************************************************
	 * Public Method 
	 ***********************************************************************/
	/**
     * Method displayServerRequestInfoMONOLOG.
     * @param info
     *  @return String 
     **/
    public String displayClientRequestInfoMonolog(org.omg.PortableInterceptor.ClientRequestInfo info) {
            return displayClientRequestInfoMonolog(info,true);
    }
    
    /**
	* Display PortableInterceptor::ClientRequestInfo MONOLOG
	* @param info 
	* @return String
	**/
	public String displayClientRequestInfoMonolog(
		org.omg.PortableInterceptor.ClientRequestInfo info,boolean withException) {
		String res = "";
		res = res + displayRequestInfo(info, logLevel);
		if (withException) res = res + displayException(info); // cause NO_RESSOURCE with OpenORB1.3.0
		if (logLevel > logLevelClientStart) {
			res = res + displayEffectiveProfile(info);
		}
		res = res + displayReceivedExceptionID(info);

		return res;
	}

	/************************************************************************
	 * Protected Method 
	 ***********************************************************************/

	/**
	 * Display PortableInterceptor::ClientRequestInfo EffectiveProfile
	 * @param info 
	 * @return String
	 **/
	protected String displayEffectiveProfile(
		org.omg.PortableInterceptor.ClientRequestInfo info) {
		org.omg.IOP.TaggedProfile profile = info.effective_profile();
		String res = "effective profile = ";
		if (profile.tag == org.omg.IOP.TAG_INTERNET_IOP.value)
			res = res + "TAG_INTERNET_IOP";
		else if (profile.tag == org.omg.IOP.TAG_MULTIPLE_COMPONENTS.value)
			res = res + "TAG_MULTIPLE_COMPONENTS";
		else
			res = res + "UNKNOWN_TAG";
		res = res + ": \n\t";
		for (int i = 0; i < profile.profile_data.length; i++)
			res = res + getOctet(profile.profile_data[i]);
		return res;
	}

	/**
	 * Display PortableInterceptor::ClientRequestInfo ReceivedExceptionID
	 * @param info 
	 * @return String
	 **/
	protected String displayReceivedExceptionID(
		org.omg.PortableInterceptor.ClientRequestInfo info) {
		String res = "";
		try {
			res =
				res
					+ "\t"
					+ "received exception id = "
					+ info.received_exception_id();
		} catch (org.omg.CORBA.BAD_INV_ORDER ex) {
			//res = res + "Operation not supported in this context";
		}
		//res = res + "\n";
		return res;
	}

    /************************************************************************
	 * Abstract implementation  Method 
     ***********************************************************************/

	protected String getLabelException() {
		return "received exception = ";
	}

	protected String displayMore(
		org.omg.PortableInterceptor.RequestInfo info) {
		return displayClientRequestInfoMonolog(
			(org.omg.PortableInterceptor.ClientRequestInfo) info);
	}

	protected org.omg.CORBA.Any requestOpException(
		org.omg.PortableInterceptor.RequestInfo info) {
                org.omg.CORBA.Any res = null;
                // ignoring Exception when NO_RESSOURCES raised 
                try{                    
                    res = ((org.omg.PortableInterceptor.ClientRequestInfo) info)
                        .received_exception();
                }catch (org.omg.CORBA.NO_RESOURCES ex){}
		return res;
	}
}
