# ====================================================================
#
# OpenCCM: The Open CORBA Component Model Platform
# Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
# Contact: openccm@objectweb.org
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
# USA
#
# Initial developer(s): Pierre Carpentier.
# Contributor(s): ______________________________________.
#
# ====================================================================
#
#  This class feeds the AST from the differents elements in the
#  UML repository. This file is build according to the UML Profile
#  for CORBA (formal/02-04-01).
#
# ====================================================================

# TO BE COMPLETED : CORBAConstant (AnyValue), fixed, arrays

import NotCreatedException;
import ASTFeeder;

class CORBA2Feeder (ASTFeeder.ASTFeeder)
{

    # =================================================================
    #   Empty Constructor for the CORBA2Feeder.
    # =================================================================
    proc __CORBA2Feeder__(self)
    {
        self.__ASTFeeder__();
    }


    # =================================================================
    #   3.5.1 - CORBA Basic Types
    # =================================================================
    proc typeOfObject (self, obj)
    {
        try {
            typename = obj.getType().getName();
            typenameLower = typename.toLowerCase();
            if (typenameLower == "short")
                return self.ast.getShortType();
            if (typenameLower == "long")
                return self.ast.getLongType();
            if (typenameLower == "long long")
                return self.ast.getLongLongType();
            if (typenameLower == "double")
                return self.ast.getDoubleType();
            if (typenameLower == "long double")
                return self.ast.getLongDouble();
            if (typenameLower == "unsigned short")
                return self.ast.getUShortType();
            if (typenameLower == "unsigned long")
                return self.ast.getULongType();
            if (typenameLower == "unsigned long long")
                return self.ast.getULongLongType();
            if (typenameLower == "any")
                return self.ast.getAnyType();
            if (typenameLower == "boolean")
                return self.ast.getBooleanType();
            if (typenameLower == "string")
                return self.ast.getStringType();
            if (typenameLower == "octet")
                return self.ast.getOctetType();
            if (typenameLower == "void")
                return self.ast.getVoidType();
            if (typenameLower == "char")
                return self.ast.getCharType();
            if (typenameLower == "wchar")
                return self.ast.getWCharType();
            if (typenameLower == "float")
                return self.ast.getFloatType();
            if (typenameLower == "wstring")
                return self.ast.getWStringType();
            if (typenameLower == "typecode")
                return self.ast.getTypeCodeType();
            if (typenameLower == "principal")
                return self.ast.getPrincipalType();
            if (typenameLower == "object")
                return self.ast.getObjectType();
            if (typenameLower == "valuebase")
                return self.ast.getValueBaseType();
            if (typenameLower == "native") {
                if (!self.typeExist(obj)) {
                    native = self.currentModule.startNative(obj.getName());
                    native.create();
                    self.putType(obj, native);
                }
                return self.getType(obj);
            }
            if (typenameLower.index("fixed") == 0) {
                token = java.util.StringTokenizer(typenameLower, "fixed<>, ");
                if (token.countTokens() == 2) {
                    digits = long._fromString(token.nextToken());
                    scale = long._fromString(token.nextToken());
                    return self.ast.createFixedType(digits, scale);
                }
            }
        } catch (exception) {
            return self.getType(obj);
        }
        return self.getType(obj.getType());
    }


    # =================================================================
    #   3.5.4 - Module
    # =================================================================
    proc createCORBAModule (self, element, currentScope)
    {
        oldPrefix = self.prefix;
        module = currentScope.declareModule(element.getName());
        self.currentModule = module;
        for tv in element.getTaggedValue().toArray() {
            if (tv.getType().getTagType() == "typePrefix" || tv.getType().getTagType() == "CORBATypePrefix") {
                self.prefix = tv.getDataValue().get(0);
            }
        }
        module.setPrefix(self.prefix);
        module.create();
        self._generated_module.append(module);
        self.create(element.getOwnedElement().toArray(), module);
        self.prefix = oldPrefix;
        self.moduleName = element.getName();
    }


    # =================================================================
    #   3.5.6 - Interface
    # =================================================================
    proc createCORBAInterface (self, element, currentScope)
    {
        try {
            if (element.isAbstract()) {
                interf = currentScope.declareAbstractInterface(element.getName());
            } else {
                isLocal = "false";
                for tv in element.getTaggedValue().toArray() {
                    if (tv.getType().getTagType() == "isLocal" || tv.getType().getTagType() == "CORBAIsLocal") {
                        isLocal = tv.getDataValue().get(0);
                    }
                }
                if (isLocal.toLowerCase() == "true") {
                    interf = currentScope.declareLocalInterface(element.getName());
                } else {
                    interf = currentScope.declareInterface(element.getName());
                }
            }
            for g in element.getGeneralization().toArray() {
                if (self.umlOperations.isStereotyped(g.getParent(), "CORBAInterface"))
                    interf.getSupportedInterfaceList().add(self.getType(g.getParent()));
            }
            interf.create();
            self.create(element.getFeature().toArray(), interf);
            self.putType(element, interf);
            # println("> Interface created : ", element.getName());
        } catch (NotCreatedException.NotCreatedException nce) {
            interf.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.7 - Value Types
    # =================================================================
    proc createCORBAValue (self, element, currentScope)
    {
        value = currentScope.declareValue(element.getName());
        if (element.isAbstract())
            value.setAbstract(true);
        self.createCORBAValueContents(value, element);
        value.create();
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBACustomValue
    # =================================================================
    proc createCORBACustomValue (self, element, currentScope)
    {
        value = currentScope.declareValue(element.getName());
        if (element.isAbstract())
            value.setAbstract(true);
        value.setCustom(true);
        self.createCORBAValueContents(value, element);
        value.create();
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValue & CORBACustomValue Contents
    # =================================================================
    proc createCORBAValueContents (self, value, element)
    {
        try {
            # generalizations
            for g in element.getGeneralization().toArray() {
                if (self.umlOperations.isStereotyped(g, "CORBAValueSupports")) {
                    self.createCORBAValueSupports(g.getParent(), value);
                } else {
                    if (g.parent().isAbstract()) {
                        value.getAbstractValueList().add(self.getType(g.getParent()));
                    } else {
                        value.setBaseValue(self.getType(g.getParent()));
                    }
                }
            }
            value.create();
            # attributes
            for statemember in element.getFeature().toArray() {
                if (statemember._is_a(org.omg.uml.core.UmlAttribute)) {
                    self.createAttribute(statemember, value, true);
                }
            }
            # attributes in association
            self.createAssociationAttribute(element, value, true);
            # operations
            for operation in element.getFeature().toArray() {
                if (operation._is_a(org.omg.uml.core.Operation)) {
                    if (self.umlOperations.isStereotyped(operation, "CORBAValueFactory")) {
                        op = value.startInitializer(operation.getName());
                        value.getInitializerList().add(op);
                    } else {
                        op = value.startOperation(operation.getName());
                        if (self.umlOperations.isStereotyped(operation, "oneway"))
                            op.setOneway();
                    }
                    self.setOperationParameters(op, operation);
                    self.setOperationExceptions(op, operation);
                    op.create();
                }
            }
            self.putType(element, value);
        } catch (NotCreatedException.NotCreatedException nce) {
            value.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValueSupports
    # =================================================================
    proc createCORBAValueSupports (self, element, value)
    {
        value.getSupportedInterfaceList().add(self.getType(element));
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBATruncatable
    # =================================================================
    proc createCORBATruncatable (self, element, value)
    {
        value.setTruncatable(true);
        value.getAbstractValueList().add(self.getType(element));
    }


    # =================================================================
    #   3.5.7 - Value Types - CORBAValueFactory
    # =================================================================


    # =================================================================
    #   3.5.8 - CORBA Wrapper Types
    # =================================================================


    # =================================================================
    #   3.5.9 - Typedef
    # =================================================================


    # =================================================================
    #   3.5.10 - Boxed Value Types
    # =================================================================
    proc createCORBABoxedValue (self, element, currentScope)
    {
        try {
            value = currentScope.declareValue(element.getName());
            if (element.isAbstract())
                value.setAbstract(true);
            value.create();
            for statemember in self.umlOperations.allAttributes(element) {
                member = value.startValueMember(statemember.getName());
                if (statemember.getVisibility() == org.omg.uml.data_types.VisibilityKindEnum.forName("vk_private"))
                    member.setPrivate()
                else
                    member.setPublic();
                member.setType(self.typeOfObject(feature));
                member.create();
            }
            self.putType(element, value);
        } catch (NotCreatedException.NotCreatedException nce) {
            value.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.11 - Constant Declaration - CORBAConstants (Class)
    # =================================================================
    proc createCORBAConstants (self, element, currentScope)
    {
        for constant in element.getFeature().toArray() {
            self.createCORBAConstant(constant, currentScope);
        }
    }


    # =================================================================
    #   3.5.11 - Constant Declaration - CORBAConstant (Attribute)
    #   TO BE COMPLETED : AnyValue
    # =================================================================
    proc createCORBAConstant (self, element, currentScope)
    {
        try {
            constant = currentScope.startConstant(element.getName());
            constant.setType(self.typeOfObject(element));
            # value = AnyValue(element.initial_value().body());
            # setAnyValue TO BE COMPLETED.
            constant.setAnyValue(CORBA.ORB.create_any());
            constant.create();
            self.putType(element, constant);
        } catch (NotCreatedException.NotCreatedException nce) {
            constant.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.14 - CORBAStruct
    # =================================================================
    proc createCORBAStruct (self, element, currentScope)
    {
        try {
            struct = currentScope.declareStruct(element.getName());
            struct.create();
            for attrib in self.umlOperations.allAttributes(element) {
                if (self.umlOperations.isStereotyped(attrib, "CORBAAnonymousSequence")) {
                    struct.getMemberList().addMember(attrib.getName(), self.getCORBAAnonymousSequence(attrib, currentScope));
                } else {
                    struct.getMemberList().addMember(attrib.getName(), self.typeOfObject(attrib));
                }
            }
            struct.create();
            self.putType(element, struct);
        } catch (NotCreatedException.NotCreatedException nce) {
            struct.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.15 - Discriminated Union
    # =================================================================
    proc createCORBAUnion (self, element, currentScope)
    {
        try {
            union = currentScope.declareUnion(element.getName());
            self.createCORBAUnionSwitch(element, union);
            self.createCORBAUnionSwitchEnd(element, union);
            union.create();
            self.putType(element, union);
        } catch (NotCreatedException.NotCreatedException nce) {
            union.destroy();
            throw nce;
        }
    }

    proc createCORBAUnionSwitch (self, element, union)
    {
        for attrib in self.umlOperations.allAttributes(element) {
            if (self.umlOperations.isStereotyped(attrib.getType(), "switch")
                || self.umlOperations.isStereotyped(attrib, "switch")) {
                union.setDiscriminator(self.typeOfObject(attrib));
                for asso in self.umlOperations.associations(element) {
                    for end in asso.getConnection().toArray() {
                        if (end.getParticipant() != element) {
                            for case in self.listCase(end.getTaggedValue().toArray()) {
                                union.getMemberList().addMember(end.getName(), self.getType(end.getParticipant()), self.caseLabels(case, attrib.getType()));
                            }
                        }
                    }
                }
            }
        }
    }

    proc createCORBAUnionSwitchEnd (self, element, union)
    {
        for end in self.umlOperations.navigableOppositeEnds(element) {
            if (self.umlOperations.isStereotyped(end.getParticipant(), "switchEnd")
                || self.umlOperations.isStereotyped(end, "switchEnd")) {
                union.setDiscriminator(self.getType(end.getParticipant()));
                union.create();
                for attrib in self.umlOperations.allAttributes(element) {
                    for case in self.listCase(attrib.getTaggedValue()) {
                        union.getMemberList().addMember(attrib.getName(), self.typeOfObject(attrib), self.caseLabels(case, end.getParticipant().toArray()));
                    }
                }
            }
        }
    }

    proc listCase(self, taggedValues)
    {
        value = "";
        listcase = array.create(0);
        for tv in taggedValues {
            if (tv.getType().getTagType() == "Case") {
                value = tv.getDataValue().get(0);
            }
        }
        token = java.util.StringTokenizer(value, "(), ");
        while (token.hasMoreTokens()) {
            listcase.append(token.nextToken());
        }
        return listcase;
    }

    proc caseLabels(self, case, switch)
    {
        typename = switch.getName().toLowerCase();
        anyValue = self.ast.createAnyValue();
        if (self.umlOperations.isStereotyped(switch, "CORBAEnum")) {
            for feat in switch.getFeature().toArray() {
                if (feat.getName() == case) {
                    anyValue.setAsEnum(self.getType(switch).getIDLType().getType(), self.getIDLOrder(feat), case);
                }
            }
        }
        if (typename == "long long")
            anyValue.setAsInteger(long._fromString(case));
        if (typename == "string")
            anyValue.setAsString(case);
        if (typename == "wstring")
            anyValue.setAsWString(case);
        if (typename == "char")
            anyValue.setAsChar(case[0]);
        if (typename == "wchar")
            anyValue.setAsWChar(case[0]);
        if (typename == "fixed")
            anyValue.setAsFixed(case);
        if (typename == "double")
            anyValue.setAsFloating(double._fromString(case));
        if (typename == "boolean") {
            if (case.toLowerCase() == "true")
                anyValue.setAsBoolean(true)
            else
                anyValue.setAsBoolean(false);
        }
        return anyValue;
    }

    proc getIDLOrder (self, feature)
    {
        for tv in feature.getTaggedValue().toArray() {
            if (tv.getType().getName() == "IDLOrder" || tv.getType().getName() == "CORBAIDLOrder") {
                value = tv.getDataValue().get(0);
                return long._fromString(value);
            }
        }
    }


    # =================================================================
    #   3.5.16 - CORBAEnum
    # =================================================================
    proc createCORBAEnum (self, element, currentScope)
    {
        try {
            enum = currentScope.startEnum(element.getName());
            list = enum.getMemberList();
            index = 0;
            for feature in element.getFeature().toArray() {
                list.add(self.getCORBAEnumElement(element, index));
                index = index + 1;
            }
            enum.create();
            self.putType(element, enum);
            # println("> Enumeration created : ", element.getName());
        } catch (NotCreatedException.NotCreatedException nce) {
            enum.destroy();
            throw nce;
        }
    }

    proc getCORBAEnumElement (self, element, idlOrder)
    {
        for feature in element.getFeature().toArray() {
            for tv in feature.getTaggedValue().toArray() {
                if (tv.getType().getName() == "IDLOrder" || tv.getType().getName() == "CORBAIDLOrder") {
                    if (tv.getDataValue().get(0) == idlOrder._toString()) {
                        return feature.getName();
                    }
                }
            }
        }
        throw NotCreatedException.NotCreatedException(element.getName() + " - IDLOrder : " + idlOrder);
    }


    # =================================================================
    #   3.5.17 - Exception
    # =================================================================
    proc createCORBAException (self, element, currentScope)
    {
        try {
            excep = currentScope.startException(element.getName());
            excep.create();
            self.create(element.getFeature().toArray(), excep);
            self.putType(element, excep);
            # println("> Exception created : ", element.getName());
        } catch (NotCreatedException.NotCreatedException nce) {
            excep.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.19 - CORBASequence
    # =================================================================
    proc createCORBASequence (self, element, currentScope)
    {
        try {
            sequence = self.ast.createSequenceType(1, self.typeOfObject(element));
            sequence.create();
            alias = currentScope.startAlias(element.getName());
            alias.setType(sequence);
            self.putType(element, alias);
            # println("> Sequence created : ", element.getName());
        } catch (NotCreatedException.NotCreatedException nce) {
            sequence.destroy();
            throw nce;
        }
    }


    # =================================================================
    #   3.5.19 - CORBAAnonymousSequence
    # =================================================================
    proc getCORBAAnonymousSequence (self, element, currentScope)
    {
        return self.ast.createSequenceType(1, self.typeOfObject(element));
    }


    # =================================================================
    #   3.5.20 - Array - CORBAArray
    # =================================================================


    # =================================================================
    #   3.5.20 - Array - CORBAAnonymousArray
    # =================================================================


    # =================================================================
    #   3.5.21 - Fixed Type
    # =================================================================


    # =================================================================
    #   3.13 - Operation Declaration
    # =================================================================
    proc createOperation (self, element, currentScope)
    {
        try {
            operation = currentScope.startOperation(element.getName());
            self.setOperationParameters(operation, element);
            self.setOperationExceptions(operation, element);
            self.setOperationContext(operation, element);
            operation.create();
            # println("> Operation created : ", element.getName());
        } catch (NotCreatedException.NotCreatedException nce) {
            operation.destroy();
            throw nce;
        }
    }



    # =================================================================
    #   3.13.2 - Operation Parameters Declaration
    # =================================================================
    proc setOperationParameters (self, operation, element)
    {
        parameterList = operation.getParameterList();
        for parameter in element.getParameter().toArray() {
            if (self.umlOperations.isStereotyped(parameter, "CORBAAnonymousSequence"))
                typeParameter = self.getCORBAAnonymousSequence(parameter, currentScope)
            else
                typeParameter = self.typeOfObject(parameter);
            if (parameter.getKind() == org.omg.uml.data_types.ParameterDirectionKindEnum.forName("pdk_return")) {
                operation.setType(typeParameter);
            } else if (parameter.getKind() == org.omg.uml.data_types.ParameterDirectionKindEnum.forName("pdk_in")) {
                parameterList.addInParam(parameter.getName(), typeParameter);
            } else if (parameter.getKind() == org.omg.uml.data_types.ParameterDirectionKindEnum.forName("pdk_inout")) {
                parameterList.addInOutParam(parameter.getName(), typeParameter);
            } else if (parameter.getKind() == org.omg.uml.data_types.ParameterDirectionKindEnum.forName("pdk_out")) {
                parameterList.addOutParam(parameter.getName(), typeParameter);
            }
        }
    }


    # =================================================================
    #   3.13.3 - Operation Raises Expressions
    # =================================================================
    proc setOperationExceptions (self, operation, element)
    {
        AContextRaisedSignalLinkSet = self.umlPackage.getCommon_Behavior().getA_context_raisedSignal().refAllLinks().toArray();
        for crsl in AContextRaisedSignalLinkSet {
            if (crsl.refFirstEnd() == element) {
                operation.getExceptionList().add(self.getType(crsl.refSecondEnd()));
            } else if (crsl.refSecondEnd() == element) {
                operation.getExceptionList().add(self.getType(crsl.refFirstEnd()));
            }
        }
    }


    # =================================================================
    #   3.13.4 - Operation Context Expressions
    # =================================================================
    proc setOperationContext (self, operation, element)
    {
        # Not present in the XMI file.
        # operation.getContextList().add(...);
    }


    # =================================================================
    #   3.14 - Attribute Declaration
    # =================================================================
    proc createAttribute (self, element, currentScope, isValueMember=false)
    {
        if (isValueMember) {
            attrib = currentScope.startValueMember(element.getName());
            if (element.getVisibility() == org.omg.uml.data_types.VisibilityKindEnum.forName("vk_private"))
                attrib.setPrivate()
            else
                attrib.setPublic();
        } else {
            attrib = currentScope.startAttribute(element.getName());
        }
        if (self.umlOperations.isStereotyped(element, "readonly"))
            attrib.setReadonly();
        if (self.umlOperations.isStereotyped(element, "CORBAAnonymousSequence"))
            attrib.setType(self.getCORBAAnonymousSequence(attrib, currentScope))
        else
            attrib.setType(self.typeOfObject(element));
        attrib.create();
        # println("> Attribute created : ", element.getName());
    }


    # =================================================================
    #   3.14 - Attribute Declaration (by Association)
    # =================================================================
    proc createAssociationAttribute (self, element, currentScope, isValueMember=false)
    {
        for asso in self.umlOperations.associations(element) {
            isAssociationAttribute = false;
            readonly = false;
            anonymousSequence = false;
            if (asso.getStereotype().size() == 0)
                isAssociationAttribute = true;
            if (self.umlOperations.isStereotyped(asso, "readonly"))
                readonly = true;
            if (self.umlOperations.isStereotyped(asso, "CORBAAnonymousSequence"))
                anonymousSequence = true;
            if (isAssociationAttribute) {
                if (asso.getConnection().get(0).getParticipant() == element) {
                    name = asso.getConnection().get(1).getName();
                    type = asso.getConnection().get(1).getParticipant();
                    visibility = asso.getConnection().get(1).getVisibility();
                } else {
                    name = asso.getConnection().get(0).getName();
                    type = asso.getConnection().get(0).getParticipant();
                    visibility = asso.getConnection().get(0).getVisibility();
                }
                if (isValueMember) {
                    attrib = currentScope.startValueMember(name);
                    if (visibility == org.omg.uml.data_types.VisibilityKindEnum.forName("vk_private"))
                        attrib.setPrivate()
                    else
                        attrib.setPublic();
                } else {
                    attrib = currentScope.startAttribute(name);
                    if (readonly) attrib.setReadonly();
                }
                if (anonymousSequence)
                    attrib.setType(self.getCORBAAnonymousSequence(type, currentScope))
                else
                    attrib.setType(self.getType(type));
                attrib.create();
                # println("> Attribute created : ", element.getName());
            }
        }
    }


}
