/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): _____________________________________.

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.URL;
import java.util.zip.ZipEntry;

import org.objectweb.ccm.descriptor.ZipEntryRetriever;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.BaseHandlerResolver;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.base.HandlerContext;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.managers.DefaultDeploymentLogger;
import org.objectweb.ccm.descriptor.componentassembly.ccm.deployer.managers.DefaultErrorManager;
import org.objectweb.ccm.descriptor.deployerconfig.Deployerconfig;
import org.objectweb.ccm.descriptor.deployerconfig.DeployerconfigImpl;
import org.objectweb.ccm.descriptor.deployerconfig.beans.DeployerconfigBeanImpl;
import org.objectweb.ccm.descriptor.softpkg.Softpkg;
import org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBean;
import org.objectweb.ccm.descriptor.softpkg.beans.SoftpkgBeanImpl;
import org.objectweb.ccm.descriptor.softpkg.ccm.ImplementationDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployer;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgDeployerHandler;
import org.objectweb.ccm.descriptor.softpkg.ccm.SoftpkgRootDeployerContext;
/**
 * This is the implementation of the ExtComponentInstallation facet 
 * provided by the NodeManager component.
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */

public class ExtComponentInstallationImpl 
            extends org.objectweb.openccm
                    .OpenCCM_DCI.NodeManagerSessionComposition
                    .ComponentInstallationSeg
{
     
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     **  To store the loaded archives.
     **/
    protected org.objectweb.ccm.util.Table implementations_;

    /**
     **  The path where the archives are stored.
     **/
    protected java.io.File path_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The constructor.
     **
     **  @param path The path where the archives are stored.
     **  @param classLoader The class loader used to load home classes.
     **/
    public
    ExtComponentInstallationImpl()
    { //  super();
        implementations_ = new org.objectweb.ccm.util.Table();
    
                
    }
    
   
        
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    public void
    setComponentInstallationDir(String path )
    {
        path_ = new java.io.File(path);
    }
    // ==================================================================
    //
    // Methods for the Components::Deployment::ComponentInstallation interface.
    //
    // ==================================================================
 
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/install:1.0
    //
    /**
     **  Install a component archive.
     **
     **  @param implUUID The implementation identifier.
     **  @param component_loc The location of the component archive.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.InvalidLocation
     **             Thrown if the location is invalid.
     **/
    public void
    install(String implUUID,
            String component_loc)
    throws org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.Deployment.InstallationFailure
    {
        // TODO: Must be fully rewritten!!!!
        
        //
        if(implementations_.contains(implUUID))
            return ;
        
        java.io.DataInputStream br= null;
        java.io.InputStream  bis = null;
        FileOutputStream bos = null;
        java.net.URL url = null;
        java.io.File output;

        
        // extraire le nom du fichier ou de l'archive pour le nom du fichier
        // de sortie.
        String filename ;
        //System.err.println("EXII>> location:"+component_loc);
        try{
            
            filename=(new java.net.URL(component_loc)).getFile();
            filename=filename.substring(filename.lastIndexOf("/"));
            
        }
        catch(Exception e0){
            e0.printStackTrace();
            throw new org.omg.Components.Deployment.InstallationFailure();
        }
        output = new java.io.File (path_, filename);
        try
            {
                path_.mkdirs();
                bos = new java.io.FileOutputStream (output);
            }
        catch (java.io.FileNotFoundException e)
            {
                // System.err.println("Fichier de sortie:"+path_+filename);
                throw new org.omg.Components.Deployment.InvalidLocation();
            }
        catch (java.io.IOException e1)
            {
                //
                //  TODO : which value to set ??
                //
                e1.printStackTrace();
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
        
        try 
            {
                url = new java.net.URL(component_loc);                        
                bis = url.openStream();                
                
            } 
        catch (java.net.MalformedURLException e)
            {               
                e.printStackTrace();
                throw new org.omg.Components.Deployment.InvalidLocation ();
            }
        catch (java.io.FileNotFoundException e)
            {
                e.printStackTrace();
                throw new org.omg.Components.Deployment.InvalidLocation();
            }
        catch (java.io.IOException e)            
            {
                //
                //  TODO : which value to set ??
                //
                e.printStackTrace();
                                
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
        
        // fetching the file
        try 
        {

            byte [] byteArray= new byte[32000];
            int readed=readed=bis.read(byteArray,0,32000);                        
            
            while(readed>0)                      
                {
                    bos.write(byteArray,0,readed);
                    readed=bis.read(byteArray,0,32000);
                }

            bis.close();
            bos.flush();
            bos.close();
        }
        catch (java.io.IOException e)
            {
                e.printStackTrace();
                                
                //
                //  TODO : which value to set ??
                //
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
        
        try{
            implementations_.put(implUUID,output.toURL().toString());
             if(component_loc.endsWith(".car")||component_loc.endsWith(".zip"))             
             {
                   processComponentArchive(output);
             }

        }
        catch(Exception ei)
            {
                ei.printStackTrace();
                System.err.println("Problem occurs during the adding of the output URL");
                throw new org.omg.Components.Deployment.InstallationFailure();
            }
        //try
        //{
        //      TheURLClassLoader.getURLClassLoader().addURL(output.toURL());
        //  }
        //  catch (java.net.MalformedURLException e)
        //  {
        //throw new org.omg.Components.Deployment.InstallationFailure();
        //  }
    }
     /**
     * @param stream
     */
    private void 
    processComponentArchive(File stream) 
    throws Exception 
    {

        ZipEntryRetriever ze=new ZipEntryRetriever(stream);        
        SoftpkgRootDeployerContext sprd=new SoftpkgRootDeployerContext();
        ze.connectStreamManager(sprd);
        sprd.connectZipEntryRetriever(ze);
        DefaultDeploymentLogger ddl=new DefaultDeploymentLogger
                                            (System.getProperty("WorkDir")
                                             +"/ComponentInstalltionLog.log");
        sprd.connectDeploymentLogger(ddl);
        sprd.connectServerSocket(new ServerSocket(0));
        //To correct adding a valid ErrorManager correctly setup

        FileReader fr=new FileReader(System.getProperty("DeployerConfigFile"));

        Deployerconfig depl_conf=(Deployerconfig) 
                                    DeployerconfigBeanImpl.unmarshal(fr,true);

        DefaultErrorManager erm=new DefaultErrorManager(depl_conf);
        erm.connectDeploymentLogger(ddl);
        sprd.connectErrorManager(erm);
        
        String csdEntryNames[]=ze.getZipEntriesEndingBy(".csd");
        InputStream is=ze.getZipEntryISIgnoringDelimiterType(csdEntryNames[0]);

        SoftpkgBeanImpl sbi=(SoftpkgBeanImpl) SoftpkgBeanImpl
                                                .unmarshalBean(new InputStreamReader(is),true);

        SoftpkgDeployerHandler softHandler=//new SoftpkgDeployerHandler();
                                    ((SoftpkgDeployerHandler)                  
                                       sbi
                                       .getExtensionManager()
                                       .getExtensionByName("ccm.Deployer"));
        
        HandlerContext dc=new HandlerContext();
        dc.connectHandlerResolver(new BaseHandlerResolver());
        softHandler.connectHandlerContext(dc);

        softHandler.connectRootDeployerContext(sprd);
        
        //connections to do
    

        SoftpkgDeployer sfpkgDeployer=(SoftpkgDeployer)softHandler.getInstance(sbi);

        sfpkgDeployer.processDependencies(this);

        ImplementationDeployer implDeployers[]=sfpkgDeployer.getImplementationDeployers();
        for(int i=0;i<implDeployers.length;i++)
            implDeployers[i].install(this);
           
        sprd.clearAllStream();
                    
    }
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/replace:1.0
    //
    /**
     **  Replace a component archive.
     **
     **  @param implUUID The implementation identifier.
     **  @param component_loc The location of the component archive.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.InvalidLocation
     **             Thrown if the location is invalid.
     **/
    public void
    replace(String implUUID,
            String component_loc)
    throws org.omg.Components.Deployment.InvalidLocation,
           org.omg.Components.Deployment.InstallationFailure
    {   
        install(implUUID, component_loc);
    }

    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/remove:1.0
    //
    /**
     **  Remove a component archive.
     **
     **  @param implUUID The implementation identifier.
     **
     **  @return True for success.
     **
     **  @exception org.omg.Components.Deployment.UnknownImplId
     **             Thrown if the implementation identifier is unknown.
     **/
    public void
    remove(String implUUID)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.RemoveFailure
    {
        //
        Object o = implementations_.remove(implUUID);
                
        //
        if(o == null)
            {
                throw new org.omg.Components.Deployment.UnknownImplId();
            }
        
        // TODO: remove the file from the disk
    }
    
    //
    // IDL:omg.org/Components/Deployment/ComponentInstallation/get_implementation:1.0
    //
    /**
     **  Get the implementation location on the local disk.
     **
     **  @param implUUID The implementation identifier.
     **
     **  @return The location of the implementation.
     **
     **  @exception org.omg.Components.Deployment.UnknownImplId
     **             Thrown if the implementation identifier is unknown.
     **/
    public String
    get_implementation(String implUUID)
    throws org.omg.Components.Deployment.UnknownImplId,
           org.omg.Components.Deployment.InstallationFailure
    {
        //
        String impl = (String)implementations_.get(implUUID);
        
        //
        if(impl == null)
            {
                throw new org.omg.Components.Deployment.UnknownImplId();
            }
        
        //
        return impl;
    }
    
    public String []
    get_all_implUUIDs()
    {
     String [] result= new String [implementations_.size()];
     int i=0;
     
     for(java.util.Enumeration enum=implementations_.keys();
         enum.hasMoreElements();){
         result[i]=(String)enum.nextElement();
         i++;
        }
                    
        return result;
    }
    
    public String
    upload( String name,byte[] data){
        // TO COMPLETE   
        return "to implement";
    }
}
