/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Components;

/** The consoles's imports */
import org.objectweb.openccm.browser.CORBA.TypageCORBA;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.omg.Components.ReceptacleDescription;
import org.omg.Components.ConnectionDescription;

/**
 * This context displays the facets connected to the given receptacle.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class ReceptacleContext 
    implements Context, Wrapper {

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The ReceptacleContainer which contains the ReceptacleDescription to browse */
    protected ReceptacleContainer receptacle_ = null;

    // ==================================================================
    //
    // No constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected String getName(org.omg.CORBA.Object o) {
        TypageCORBA tc = new TypageCORBA(o, org.objectweb.ccm.CORBA.TheORB.getORB());
        return tc.getTypeID();
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        ReceptacleDescription r = (ReceptacleDescription) receptacle_.getReceptacle();
        ConnectionDescription[] cd = r.connections;
        Entry[] values = new Entry[cd.length];
        for (int i = 0; i < cd.length; i++) {
            ConnectionContainer cc = new ConnectionContainer();
            cc.setConnection(cd[i]);
            cc.setReceptacleContainer(receptacle_);
            values[i] = new DefaultEntry(cc, new DefaultName("Connection (" + i + ")"), this);
        }
        return values;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     * @param object The object to wrap
     */
    public void setWrapped(Object object) {
        receptacle_ = (ReceptacleContainer) object;
    }

    /**
     * Gets the wrapped object
     * @return The wrapped object
     */
    public Object getWrapped() {
        return receptacle_;
    }

}
