/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.DCI;

/** The consoles's imports */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.core.naming.DefaultEntry;
import org.objectweb.util.browser.core.naming.DefaultName;

/** The OpenCCM platform imports */
import org.objectweb.openccm.OpenCCM_DCI.AssemblyManager;
import org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManager;
import org.objectweb.openccm.OpenCCM_DCI.AssemblyFactoryManagerPackage.assembly_managersConnection;

/**
 * This context displays the <code>AssemblyFactory</code> and the <code>ComponentInstallation</code> registered in the given NodeManager.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class AssemblyFactoryManagerContext 
  implements Context, Wrapper 
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /* The AssemblyFactoryManager to browse */
    protected AssemblyFactoryManager assemblyFactoryManager_ = null;

    // ==================================================================
    //
    // No constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Provides a name depending on the AssemblyManager id.
     */
    protected String getName(AssemblyManager assemblyManager){
        String returnedValue = "Assembly ";
        String assemblyId = assemblyManager.ass_id();
        if(assemblyId.indexOf(":")!=-1)
            returnedValue = assemblyId.substring(0,assemblyId.indexOf(":"));
        return returnedValue;
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {
        java.util.List v = new java.util.Vector();

        assembly_managersConnection[] assemblyManagerList = assemblyFactoryManager_.get_connections_assembly_managers();
        for (int i=0 ; i<assemblyManagerList.length ; i++){
            assembly_managersConnection assemblyManagerConnection = assemblyManagerList[i];
            AssemblyManagerWrapperWithCookie assemblyManagerWrapper = new AssemblyManagerWrapperWithCookie();
            assemblyManagerWrapper.setAssemblyFactory(assemblyFactoryManager_.provide_assembly_factory());
            assemblyManagerWrapper.setCookie(assemblyManagerConnection.ck);
            assemblyManagerWrapper.setAssemblyManager(assemblyManagerConnection.objref);
            v.add(new DefaultEntry(assemblyManagerWrapper, new DefaultName(getName(assemblyManagerConnection.objref)+ " (" + (i+1) + ")")));
        }
        
        // Return the array of entry
        return (Entry[]) v.toArray(new Entry[0]);
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     * @param object The object to wrap
     */
    public void setWrapped(Object object) {
        assemblyFactoryManager_ = (AssemblyFactoryManager)object;
    }

    /**
     * Gets the wrapped object
     * @return The wrapped object
     */
    public Object getWrapped() {
        return assemblyFactoryManager_;
    }

}
