/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.objectweb.openccm.browser.util.ior;

final class ModestOutputStream  {
    private byte bytes[];
    private int index;
    private boolean little_endian;
    private void AddBufferSize(int add) {
        if (index + add > bytes.length) {
            if (add < 256) {
                add = 256;
            }
            byte b[] = new byte[bytes.length + add];

            System.arraycopy(bytes, 0, b, 0, index);
            bytes = b;
        }
    }

    public ModestOutputStream(boolean little_endian) {

        this.little_endian = little_endian;
        bytes = new byte[1024];
        bytes[0] = (byte)(little_endian ? 1 : 0);
        index = 1;
    }

    public void WriteOctet(byte octet) {
        AddBufferSize(1);
        bytes[index++] = octet;
    }

    public void WriteUShort(short value) {
        int al = index % 2;
        AddBufferSize(al + 2);
        index += al;

        if (little_endian) {
            bytes[index++] = (byte)value;
            bytes[index++] = (byte)(value >> 8);
        } else {
            bytes[index++] = (byte)(value >> 8);
            bytes[index++] = (byte)value;
        }
    }

    public void WriteULong(int value) {
        int al = index % 4;

        if (al != 0) {
            al = 4 - al;
            AddBufferSize(al + 4);
            index += al;
        } else {
            AddBufferSize(4);
        }

        if (little_endian) {
            bytes[index++] = (byte)value;
            bytes[index++] = (byte)(value >> 8);
            bytes[index++] = (byte)(value >> 16);
            bytes[index++] = (byte)(value >> 24);
        } else {
            bytes[index++] = (byte)(value >> 24);
            bytes[index++] = (byte)(value >> 16);
            bytes[index++] = (byte)(value >> 8);
            bytes[index++] = (byte)value;
        }
    }

    public void WriteString(String str) {
        byte b[] = str.getBytes();
        int len = b.length;
        WriteULong(len + 1);
        AddBufferSize(len);
        System.arraycopy(b, 0, bytes, index, len);
        index += len;
        WriteOctet((byte)0);
    }

    public void WriteOctetArray(byte b[]) {
        int len = b.length;
        WriteULong(len);
        AddBufferSize(len);
        System.arraycopy(b, 0, bytes, index, len);
        index += len;
    }

    public byte[] GetBytes() {
        byte b[] = new byte[index];
        System.arraycopy(bytes, 0, b, 0, index);
        return b;
    }
}
