/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.icon;

/** To use the SimpleContextFactory interface */
import org.objectweb.util.browser.api.IconFactory;

/** To use the ImageIcon class */
import javax.swing.ImageIcon;

/** To use the Icon interface */
import javax.swing.Icon;

/** To use the URL class */
import java.net.URL;

/**
 * Basic implementation for Icon factories.
 *
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */

public class DefaultIconFactory implements IconFactory {

    /** The Associated ImageIcon */
    protected Icon icon_ = null;

    /**
     * Return an ImageIcon associated to the param's object.
     *
     * @param object The object you want to find the associated icon
     * @return The associated icon.
     */
    public Icon newIcon(Object object) {
        return icon_;
    }

    /**
     * Fix the url of the icon file to display
     * 
     * @param url The url of the file to display
     */
    public void setUrl(String url) {
        if (url != null) {
            try {
                URL urlFile = new URL(url);
                icon_ = new ImageIcon(urlFile);
            } catch (java.net.MalformedURLException e) {
                System.out.println(url + " : Malformed URL !");
                //	    }catch(java.io.IOException e){
                //		e.printStackTrace();
            }
        }
    }

}
