/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.wrapper;

/** To use the AssociatedContext interface */
import org.objectweb.util.browser.api.AssociatedContext;

/** To use the PropertyLoader interface */
import org.objectweb.util.browser.api.BrowserProperty;

/** To use the Context interface */
import org.objectweb.util.browser.naming.api.Context;

/** To use the Entry interface */
import org.objectweb.util.browser.naming.api.Entry;

/** To use the ContextFactory interface */
import org.objectweb.util.browser.naming.api.ContextFactory;

/** To use the AssociatedElementImpl class */
import org.objectweb.util.browser.gui.common.AssociatedElementImpl;

/**
 * Implementation of AssociatedContext API
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class AssociatedContextImpl
    extends AssociatedElementImpl
    implements AssociatedContext {

    /** 
     * Default constructor
     * @param loader The propertyLoader to use to load new configuration
     */
    public AssociatedContextImpl(BrowserProperty loader) {
        super();
        context_ = new WrapperPropertyContainer(loader);
    }

    /** 
     * Returns the context associates to the node
     *
     * @param node The node 
     * @return The associated context or null if no context is found
     */
    public Context getContext(Object node) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null) {
                ContextFactory cf = (ContextFactory) entry.getOWValue();
                return cf.newOWContext(node);
            }
        }
        return null;
    }

}
