/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import org.objectweb.util.cpp.api.Preprocessor;

/**
 * This is an implementation for -I preprocessor option, e.g.
 * -IDIR.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class OptionIDIR
     extends PreprocessorOptionBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param preprocessor The associated preprocessor.
     */
    public
    OptionIDIR(Preprocessor preprocessor)
    {
        setLabels(new String[] { "-IDIR" });
        setDescription(new String[] { "Put DIR in the include file search path"});
        setPreprocessor(preprocessor);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorHolder
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOption
    //
    // ==================================================================

    /**
     * Gets the base option label.
     *
     * @return The base option label.
     */
    public String
    getBaseOptionLabel()
    {
        return "-I";
    }  

    /**
     * Consumes the option value.
     *
     * @param current The option value.
     */
    public void
    consumeOption(String current)
    {
        // Adds the DIR to the include directories of the preprocessor.
        getPreprocessor().addIncludeDirectory(current);
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
