@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
rem # Contact: openccm@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Marc Navarro (mnavarro@wins.uva.nl),
rem #                       Philippe Merle.
rem # Contributor(s): Christophe Demarey.
rem #
rem # ====================================================================
rem #
rem # Starting the OpenCCM Interface Repository.
rem #
rem # ====================================================================

setlocal

rem # Check if the OpenCCM Interface Repository is already started.
if not exist %OpenCCM_CONFIG_DIR%\IR3.PID goto not_started

echo Error: The OpenCCM Interface Repository is already started!
goto :eof

:not_started

echo The OpenCCM Interface Repository will be started.

rem # Check if the OpenCCM configuration directory doesn't exist.
if exist %OpenCCM_CONFIG_DIR% goto cfg_exist

echo Creating the %OpenCCM_CONFIG_DIR% directory.
mkdir %OpenCCM_CONFIG_DIR%

:cfg_exist

echo Launching the OpenCCM Interface Repository.

rem # Remove the IR3.start file if present.
if exist %OpenCCM_CONFIG_DIR%\IR3.start del %OpenCCM_CONFIG_DIR%\IR3.start

rem # For the older OpenCCM Interface Repository.
rem set args="%OpenCCM_CONFIG_DIR%\IR3.IOR %OpenCCM_HOMEDIR%/xml/launcher/IR3Install.xml "
rem call launcher fork "" %args% %OpenCCM_CONFIG_DIR%\IR3.output %OpenCCM_CONFIG_DIR%\IR3.PID

rem # For the newer OpenCCM Interface Repository.
set args="--ior-file %OpenCCM_CONFIG_DIR%\IR3.IOR %* %OpenCCM_HOMEDIR%/xml/launcher/IR3Server.xml "
call launcher fork "" %args% %OpenCCM_CONFIG_DIR%\IR3.output %OpenCCM_CONFIG_DIR%\IR3.PID

rem # Wait for the IR3.IOR file produced.
:wait
set READY=
if not exist %OpenCCM_CONFIG_DIR%\IR3.IOR goto :wait
copy %OpenCCM_CONFIG_DIR%\IR3.IOR %OpenCCM_CONFIG_DIR%\IR3.IOR.tmp >nul
for /f %%i in (%OpenCCM_CONFIG_DIR%\IR3.IOR.tmp) do set READY=ok
if not defined READY goto :wait
del %OpenCCM_CONFIG_DIR%\IR3.IOR.tmp

echo Feeding the OpenCCM Interface Repository with the IFR_3_0.idl file.
call ir3_feed %OpenCCM_HOMEDIR%\idl\IFR_3_0.idl

echo Feeding the OpenCCM Interface Repository with the Components.idl file.
call ir3_feed %OpenCCM_HOMEDIR%\idl\Components.idl

rem # Unblock the Ant compilation when waiting for
rem # the OpenCCM Interface Repository is started.
echo IR3_STARTED > %OpenCCM_CONFIG_DIR%\IR3.start

echo The OpenCCM Interface Repository is started.
