/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s):

====================================================================*/

package org.objectweb.ccm.descriptor.softpkg.gui;

import org.objectweb.ccm.descriptor.softpkg.beans.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.corbacomponent.beans.CorbacomponentBean;
import org.objectweb.ccm.descriptor.corbacomponent.beans.CorbacomponentBeanImpl;
import org.objectweb.ccm.descriptor.corbacomponent.gui.CorbacomponentZipper;

/**
 * This is an extension to the Apollon generated framework around softpkg DTD
 *  it looks for subelements that could contain files to archive
 *  and calls their own zipper extensions to do the same job
 *  any found file name is sent to a zip factory
 *
 */
public class DescriptorZipper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     *  The zip method called from parents
     *
     */
    public void
    zipCall(
        DescriptorBean descriptor
        , org.objectweb.openccm.deployment.gui.ZipFactory zip_factory
        )
    {
        FileinarchiveBean fileinarchive
         = (FileinarchiveBean)
            descriptor.getFileinarchive();

        // goes through sub element and call its zip method
        if (fileinarchive != null)
        {
            FileinarchiveZipper sub_zipper
             = (FileinarchiveZipper) fileinarchive
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(fileinarchive, zip_factory);


            // unmarshal the pointed CCD file to find dependencies
            java.io.File ccd_file
                = new java.io.File(fileinarchive.getName());

            if (ccd_file.exists())
            {
                try
                {
                    CorbacomponentBean ccd_instance
                        = CorbacomponentBeanImpl.unmarshalBean(ccd_file);

                    CorbacomponentZipper ccd_zipper
                        = (CorbacomponentZipper)
                            ccd_instance
                                .getExtensionManager()
                                .getExtensionByName("gui.Zipper")
                                .getInstance();

                    ccd_zipper.zipCall(ccd_instance, zip_factory);

                } catch (java.io.IOException exc) {
                    System.err.println(
                        "CCD file: "
                        +fileinarchive.getName()
                        +" could not be opened -> no CORBA properties dependencies to add.");
                }
            }
        }
    }
}
