/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.deployment.gui;

import java.io.*;
import java.util.zip.*;

/**
 * This is the factory that can build zip archives from files added
 *
 * @author <a href="mailto:Christophe.Contreras@lifl.fr">Christophe Contreras</a>
 *
 */
public class ZipFactory
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** the files to zip list */
    private java.util.List files_to_zip_;

    /** the name of archive */
    private String zipname_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    ZipFactory()
    {
        // inits the files list
        files_to_zip_ = new java.util.LinkedList();

    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * compress given file with intern dir name into the zip stream
     *
     * @param File inputfile the file to zip
     * @param ZipOutputStream zos the stream
     */
    private void
    compressFile(File inputfile, ZipOutputStream zos)
        throws Exception
    {
        byte[] buff = new byte[2048];
        FileInputStream fis = new FileInputStream(inputfile);

        int ret = 0;

        zos.putNextEntry(new ZipEntry(inputfile.getPath()));
        while (ret!=-1) {
            ret = fis.read(buff, 0, 2048);
            if (ret!=-1) {
                zos.write(buff, 0, ret);
            }
        }

        zos.closeEntry();
        fis.close();
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * add method to give the factory a new file to archive
     *
     * @param String filename the filename that will be archived
     */
    public void
    addFileToZip(String filename)
    {
        if (! files_to_zip_.contains(filename))
        {
            files_to_zip_.add(filename);
        }
    }

    /**
     * the set method for the zip archive filename
     *
     * @param String name the file name to use
     */
    public void
    setZipName(String name)
    {
        this.zipname_ = name;
    }

    /**
     * build method for the generation of the zip file
     *
     */
    public void
    buildZip()
        throws Exception
    {
        // get each file added to the private list
        String[] files
            = (String[]) files_to_zip_.toArray(new String[0]);

        // inits the output stream
        FileOutputStream fos = new FileOutputStream(zipname_);
        ZipOutputStream zos = new ZipOutputStream(fos);

        // adds the file to the stream
        for (int i=0;i<files.length;i++)
        {

            // tests if the file can be found
            java.io.File file_to_add
             = new java.io.File (
                files[i]
                );

            if ( file_to_add.exists() )
            {
                System.err.println("Adding file: "+file_to_add.getPath()+" to zip");
                compressFile(
                    file_to_add,
                    zos
                    );
            } else {
                System.err.println(
                    files[i]
                    +" can not be found -> skipping.."
                    );
            }
        }

        zos.close();
    }
}

