/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.ast.lib;

/**
 * CidlScopeImpl is a wrapper class for all CIDL definition scopes.
 *
 * Inherits from:
 * - Scope as CIDL scopes are also IDL scope.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

abstract public class CidlScopeImpl
                extends ScopeImpl
                implements org.objectweb.openccm.ast.api.CidlScope
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the declaration.
     */
    protected
    CidlScopeImpl(Repository rep,
              ScopeImpl parent)
    {
        // Call the DeclarationImpl constructor.
        super(rep, parent);

        // Init internal state.
        content_loaded_ = true;
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Create the declaration.
     */
    public void
    create()
    {
        if (the_parent_!=null)
            the_parent_.addDecl(this);
    }

    /**
      * Obtain its category (IDL, PSDL, CIDL).
      *
      * @return The category of the declaration.
      */
    public org.objectweb.openccm.ast.api.DeclarationCategory
    getCategory()
    {
        return org.objectweb.openccm.ast.api.DeclarationCategory.dc_cidl;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Scope
    //
    // ==================================================================

}
