/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.command.lib;

// Package dependencies.
import org.objectweb.openccm.command.api.CommandOnIR3;
import org.objectweb.util.cmdline.api.CommandLine;

/**
 * Abstract base class for commands using the OpenCCM Interface Repository.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.3
 */

abstract public class CommandOnIR3Base
              extends ApplicationBase
           implements CommandOnIR3
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The OpenCCM Interface Repository. */
    private org.objectweb.openccm.ccm.ComponentRepository componentRepository_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    CommandOnIR3Base()
    {
        // Calls the ApplicationBase constructor.
        super();

        // Inits internal state.
        componentRepository_ = null;
    }

    /**
     * The constructor with the command line manager.
     *
     * @param commandLine The command line manager.
     */
    public
    CommandOnIR3Base(CommandLine commandLine)
    {
        // Calls the ApplicationBase constructor.
        super(commandLine);

        // Inits internal state.
        componentRepository_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnIR3
    //
    // ==================================================================

    /**
     * Checks availability of the OpenCCM Interface Repository.
     *
     * @return True if ok, else false.
     */
    public boolean
    checkComponentRepository()
    {
        try
        {
            // Obtains the OpenCCM Interface Repository.
            componentRepository_ = org.objectweb.openccm.ccm.ComponentRepositoryHelper.narrow(
                                   org.objectweb.openccm.corba.TheInterfaceRepository.getRepository());
        }
        catch(Exception ex)
        {
            getConsole().error("Problem to obtain the OpenCCM Interface Repository reference!");
            report_exception(ex);
            return false;
        }
        return true;
    }

    /**
     * Obtains the OpenCCM Interface Repository reference.
     *
     * @return The OpenCCM Interface Repository reference.
     */
    public org.objectweb.openccm.ccm.ComponentRepository
    getComponentRepository()
    {
        if(componentRepository_ == null) {
            checkComponentRepository();
        }
        return componentRepository_;
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
