/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.generator.java.ast.lib;

import org.objectweb.openccm.generator.java.ast.api.AbstractObject;
import org.objectweb.openccm.generator.java.ast.api.InterfaceObject;
import org.objectweb.openccm.generator.java.ast.api.MethodObject;
import java.util.ArrayList;

/**
 * This is the representation for java Interfaces.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class InterfaceObjectImpl
     extends ObjectWithPackageImpl
  implements InterfaceObject
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The delegation for an abstract */
    private AbstractObject abstract_delegate_;

    /** Interface Methods */
    private ArrayList methods_;

    /** Interface import statements */
    private ArrayList imports_;

    /** Implemented objects */
    private ArrayList inherited_objects_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    InterfaceObjectImpl()
    {
        // Init internal states
        super();
        abstract_delegate_ = new AbstractObjectImpl();
        methods_ = new ArrayList();
        imports_ = new ArrayList();
        inherited_objects_ = new ArrayList();
    }

    /**
     * The constructor with the name.
     *
     * @param name - The name to set.
     */
    public
    InterfaceObjectImpl(String name)
    {
        // Init internal states
        super();
        abstract_delegate_ = new AbstractObjectImpl();
        methods_ = new ArrayList();
        imports_ = new ArrayList();
        inherited_objects_ = new ArrayList();

        setName(name);
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.generator.java.ast.api.AbstractObject
    //
    // ==================================================================

    /**
     * Set abstract this object.
     *
     * @param b - The value to set.
     */
    public void
    setAbstract(boolean b)
    {
        abstract_delegate_.setAbstract(b);
    }

    /**
     * Is this object abstract ?
     *
     * @return True if it is abstract, else false.
     */
    public boolean
    isAbstract()
    {
        return abstract_delegate_.isAbstract();
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.generator.java.ast.api.InterfaceObject
    //
    // ==================================================================

    /**
     * Add an import statement.
     *
     * @param object - The object to add.
     */
    public void
    addImport(String object)
    {
        imports_.add(object);
    }

    /**
     * Obtain all import statements.
     *
     * @return A list of import statements.
     */
    public ArrayList
    getImports()
    {
        return imports_;
    }

    /**
     * Add an implemented object.
     *
     * @param object - The object to add.
     */
    public void
    addInheritedObject(String object)
    {
        inherited_objects_.add(object);
    }

    /**
     * Obtain all implemented objects.
     *
     * @return A list of implemented objects.
     */
    public ArrayList
    getInheritedObjects()
    {
        return inherited_objects_;
    }

    /**
     * Add a method.
     *
     * @param method - The method to add.
     */
    public void
    addMethod(MethodObject method)
    {
        methods_.add(method);
    }

    /**
     * Obtain all methods.
     *
     * @return A list of methods.
     */
    public ArrayList
    getMethods()
    {
        return methods_;
    }
}
