/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.generator.java.ast.lib;

/**
 * Enumeration for modifiers.
 *
 * @author  <a href="mailto:Christophe.Demarey * @lifl .fr">Christophe Demarey</a>
 *
 * @version  0.1
 */
public final class ModifierKindImpl
{
    // ==================================================================
    //
    // Class Internal State.
    //
    // ==================================================================

    /**
     * Enum member mk_public value
     */
    public static final int mk_public_ = 0;

    /**
     * Enum member mk_public
     */
    public static final ModifierKindImpl mk_public = new ModifierKindImpl(mk_public_);

    /**
     * Enum member mk_protected value
     */
    public static final int mk_protected_ = 1;

    /**
     * Enum member mk_protected
     */
    public static final ModifierKindImpl mk_protected = new ModifierKindImpl(mk_protected_);

    /**
     * Enum member mk_private value
     */
    public static final int mk_private_ = 2;

    /**
     * Enum member mk_private
     */
    public static final ModifierKindImpl mk_private = new ModifierKindImpl(mk_private_);


    // ==================================================================
    //
    // Internal State.
    //
    // ==================================================================

    /**
     * Internal member value
     */
    private final int ModifierKind_value_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * Private constructor
     * @param  the enum value for this new member
     */
    private ModifierKindImpl( int value )
    {
        ModifierKind_value_ = value;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Return the internal member value
     * @return the member value
     */
    public int value()
    {
        return ModifierKind_value_;
    }

    /**
     * Return a enum member from its value
     * @param  an enum value
     * @return an enum member
     */
    public static ModifierKindImpl from_int(int value)
    {
        switch (value)
        {
        case 0 :
            return mk_public;
        case 1 :
            return mk_protected;
        case 2 :
            return mk_private;
        }
        return null;
    }

    /**
     * Return a string representation
     * @return a string representation of the enumeration
     */
    public java.lang.String toString()
    {
        switch (ModifierKind_value_)
        {
        case 0 :
            return "public";
        case 1 :
            return "protected";
        case 2 :
            return "private";
        }
        return "";
    }

}
