/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

// Package dependencies
import java.util.Map;

/**
 * This task feeds a PSDL file in the OpenCCM Interface Repository
 * and generates only PSDL declarations.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class PSDLtoPSDL
     extends CompilerBase
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The output file name. */
    private String dest_file_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param project_properties - Properties of the Ant Project.
     */
    public PSDLtoPSDL(Map project_properties)
    {
        // Call CompilerBase constructor
        super(project_properties);
        
        // Init internal state
        dest_file_ = "generated.psdl";
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Configure the task.
     */
    public void
    configure()
    {
        // Call the CompilerBase configure method
        super.configure();
        
        setXmlFile( project_properties_.getProperty("OpenCCM_launcher_dir") + "/PSDLtoPSDL.xml" );

        // Add destination file
        addArgument("-o");
        addArgument(dest_file_);
    }
    
    /**
     * Set Java mapping output directory.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    setDestfile(String filename)
    {
        dest_file_ = filename;
    }
    
}
