/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: OptionArg.java,v 1.1 2004/02/05 20:29:57 rouvoy Exp $
====================================================================*/

package org.objectweb.util.launcher.option;


import org.objectweb.util.cmdline.api.Iterator;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.launcher.CommandFactory;


/**
 * Definition of the --runid option.<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application.
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class OptionArg
     extends DefaultOptionArgument
{
    /** Short Tag Description */
    public final static String shortTag = "-arg" ;
    /** Long Tag Description */
    public final static String longTag = "--argument" ;

    /**
     * Default Constructor
     */
    public OptionArg(){
        super(new String[]{shortTag, longTag},
              "<Argument>",
              "The value of the argument",
              "");
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void consume(Iterator iterator) {
        setArgument(consumeArgument(iterator));
        CommandFactory.instance().getDefaultArguments().add(getArgument());
    }

    /**
     * Creates a commandline representation of the argument.
     * @param value value of the argument.
     * @return commandline representation of the argument.
     */
    public static String create(String value) {
        return shortTag+" "+value+" " ; 
    }
}
