/*====================================================================
Tnis file was produced by the OpenCCM ir3_jimpl generator.
OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.
You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Sylvain Leblanc_______________________.
====================================================================*/
package ist.coach.coachEmfServicesComponents;

import java.util.LinkedList;
import java.util.ListIterator;

import org.opennms.protocols.snmp.*;

import ist.coach.coachEmfCommon.Utils;
import ist.coach.coachEmfCommon.ExceptionMessages;
import ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError;
import ist.coach.coachEmfCommon.DataTypeImpl;
import ist.coach.coachEmfServices.SnmpAdapter.DataType;
import ist.coach.coachEmfServicesComponents.SnmpAdapterProviderPackage.trap_targetConnection;

import org.omg.CORBA.Any;
import org.omg.CosNaming.NameComponent;
/**
 **  Implementation skeleton class for the ::coachEmfServicesComponents::SnmpAdapterProvider component.
 **/
public class SnmpAdapterProviderMonolithicImpl
       extends org.omg.CORBA.LocalObject
       implements CCM_SnmpAdapterProvider,
                  ist.coach.coachEmfServices.SnmpAdapter.CCM_SnmpConnector,
                  ist.coach.coachEmfServices.SnmpAdapter.CCM_SnmpTrapDaemon,
                  org.omg.Components.SessionComponent,
		  			Runnable,
		  SnmpTrapHandler
{
    // ==================================================================
    //
    // Internal State.
    //
    // ===================================================================
	trap_targetConnection[] trap_target_facades = null;

    /**
     **  Context reference.
     **/
    private CCM_SnmpAdapterProvider_Context the_context_;
    /**
     ** Private ORB reference (used to create Anys)
     **/
    private org.omg.CORBA.ORB orb;
    /**
     ** Linked List used by SnmpTrapHandler to store registered objects.
     **/
    private LinkedList linkedList;
    /**
     ** Configuration Object used by SnmpTrapDaemon
     **/
    private Configuration config;
    /**
     ** Declaration of the identifier used to identify SNMP trap types
     **/
    private static final SnmpObjectId SNMPv2_MIB_snmpTrapOid =
	new SnmpObjectId(".1.3.6.1.6.3.1.1.4.1.0");

	public static final int SNMP_TRAP_PORT = 2010;
    /**
     ** SnmpTrapDaemon Listener Thread
     **/
    private Thread listenerThread;
    /**
     ** Declaration of Unknown Trap
     **/
    private static final int UNKNOWN_TRAP_TYPE = 0;
   // ==================================================================
    //
    // Constructor.
    //
    // ===================================================================
    /**
     **  The default constructor.
     **/
    public
    SnmpAdapterProviderMonolithicImpl()
    {
        the_context_ = null;
		this.linkedList = new LinkedList();

    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ===================================================================

    private void listener_init() {
		if (listenerThread == null) {
			listenerThread = new Thread(this, "Listener");
			listenerThread.start();
		}
    }

    private void listener_stop() {

		Thread dummyThread =listenerThread;
		listenerThread = null;
	    dummyThread.interrupt();

    }

    private Any convertSnmpSyntaxToAny(SnmpSyntax data) {

		Any value = this.orb.create_any();


		switch (data.typeId()) {
			case SnmpSMI.SMI_OBJECTID:
				value.insert_string( new String(((SnmpObjectId)data).toString()));
				break;
			case SnmpSMI.SMI_COUNTER32:
				value.insert_longlong( ((SnmpCounter32)data).getValue());
				break;
			case SnmpSMI.SMI_COUNTER64: //Same as SnmpSMI.SMI_UNSIGNED32
				value.insert_longlong( ((SnmpCounter64)data).getValue().longValue());
				break;
			case SnmpSMI.SMI_GAUGE32:
				value.insert_longlong( ((SnmpGauge32)data).getValue());
				break;
			case SnmpSMI.SMI_INTEGER:
				value.insert_long( ((SnmpInt32)data).getValue());
				break;
			case SnmpSMI.SMI_TIMETICKS:
				value.insert_longlong( ((SnmpTimeTicks)data).getValue());
				break;
			case SnmpSMI.SMI_IPADDRESS:
			    value.insert_string( new String(((SnmpIPAddress)data).toString()));
				break;
			case SnmpSMI.SMI_STRING:
			    //fset HACK
				// It seems that orbacus has problems handling anys as strings if these contain
				// non printable characters. NOT RESOLVED BUT BYPASSED....
			    //System.err.println("SnmpSMI.SMI_STRING length= " + ((SnmpOctetString)data).getLength());
				//System.err.print("As string = " + ((SnmpOctetString)data).toString() +
				//    " as bytes = " + new String(((SnmpOctetString)data).getString()));
				/*********
				if (octetData.toString().indexOf("RAW") < 0) {
				  //System.err.println(" Type is HEX");
				  value.insert_string("Hello");
				 }
				 else {
				  //System.err.println(" Type is RAW");
				  value.insert_string( new String(((SnmpOctetString)data).getString()));
			  	}
				******/
				SnmpOctetString octetData = (SnmpOctetString) data;
				byte[] octetDataBytes = octetData.getString();
				org.omg.CORBA.OctetSeqHelper.insert(value, octetDataBytes);

				//initially by digi
				//value.insert_string( new String(((SnmpOctetString)data).getString()));
				break;
			case SnmpSMI.SMI_OPAQUE:
			    value.insert_string( new String(((SnmpOpaque)data).getString()));
				break;
			case SnmpSMI.SMI_NOSUCHOBJECT:
				//value = null;
				// Is this good...???
				value.insert_string("No Such Object");
				break;
			case SnmpSMI.SMI_NOSUCHINSTANCE:
				//value = null;
				// Is this good...???
				value.insert_string( "No Such Instance");
				break;
			default:
				System.err.println("Unknown Variable Binding.");
				value = null;
				break;

		}

		return value;

    }

    private SnmpSyntax convertAnyToSnmpSyntax(Any data, byte code) {

		SnmpSyntax value = null;

		switch (code) {

			case SnmpSMI.SMI_COUNTER32:
				value = ((SnmpSyntax)new SnmpCounter32(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_COUNTER64:
				value = ((SnmpSyntax)new SnmpCounter64(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_UNSIGNED32: // same as SnmpSMI.SMI_GAUGE32
				value = ((SnmpSyntax)new SnmpGauge32(data.extract_longlong()));
				break;
			case SnmpSMI.SMI_INTEGER:
				value = ((SnmpSyntax)new SnmpInt32(data.extract_long()));
				break;
			case SnmpSMI.SMI_IPADDRESS:
				value = ((SnmpSyntax)new SnmpIPAddress(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_OBJECTID:
				value = ((SnmpSyntax)new SnmpObjectId(data.extract_string()));
				break;
			case SnmpSMI.SMI_OPAQUE:
				value = ((SnmpSyntax)new SnmpOpaque(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_STRING:
			    value = (SnmpSyntax)new SnmpOctetString(org.omg.CORBA.OctetSeqHelper.extract(data));
				//initially by digi
				//value = ((SnmpSyntax)new SnmpOctetString(data.extract_string().getBytes()));
				break;
			case SnmpSMI.SMI_TIMETICKS:
				value = ((SnmpSyntax)new SnmpTimeTicks(data.extract_longlong()));
				break;
			default:
				System.err.println("SnmpAdapter:convertAnyToSnmpSyntax> " +
				    " Unknown (or un-supported) SMI Type");
				break;
		}

		return value;
    }

    private void processTrapData(
		java.net.InetAddress agent,
		SnmpOctetString community,
		SnmpPduPacket pdu)
    throws SnmpApplicationError{

		int snmpTrapType = 0;
		String trapObjectIdentifier = null;
		int k = pdu.getLength();
		DataTypeImpl[] dataSetType = new DataTypeImpl[k];

		for (int i = 0; i < k ; i++ ) {

			SnmpVarBind vb = pdu.getVarBindAt(i);

			String oid = vb.getName().toString();
			Any value = this.orb.create_any();

			switch (vb.getValue().typeId()) {
				case SnmpSMI.SMI_OBJECTID:
					value.insert_string( ((SnmpObjectId)vb.getValue()).toString());
					break;
				case SnmpSMI.SMI_COUNTER32:
					value.insert_longlong( ((SnmpCounter32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_COUNTER64:
					value.insert_longlong( ((SnmpCounter64)vb.getValue()).getValue().longValue());
					break;
				case SnmpSMI.SMI_GAUGE32:
					value.insert_longlong( ((SnmpGauge32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_INTEGER:
					value.insert_long( ((SnmpInt32)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_TIMETICKS:
					value.insert_longlong( ((SnmpTimeTicks)vb.getValue()).getValue());
					break;
				case SnmpSMI.SMI_IPADDRESS:
					value.insert_string( ((SnmpIPAddress)vb.getValue()).toString());
					break;
				case SnmpSMI.SMI_STRING:
					SnmpOctetString octetData = (SnmpOctetString) vb.getValue();
				    byte[] octetDataBytes = octetData.getString();
				    org.omg.CORBA.OctetSeqHelper.insert(value, octetDataBytes);
					//by digi
					//value.insert_string( new String(((SnmpOctetString)vb.getValue()).getString()));
					break;
				default:
					System.err.println("SnmpAdapter: processTrapData> Unknown Variable Binding.");
					break;

			}

			if (vb.getName().compare(SNMPv2_MIB_snmpTrapOid) == 0)
				trapObjectIdentifier = ((SnmpObjectId)vb.getValue()).toString();

			dataSetType[i] = new DataTypeImpl(oid, value);

		}

		snmpTrapType = determineTrapType(trapObjectIdentifier);
		if (snmpTrapType == UNKNOWN_TRAP_TYPE)
			throw new SnmpApplicationError(ExceptionMessages.uknown_trap_error);

		String objectClass = determineObjectClass(snmpTrapType);
		if (objectClass == null) {
			System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to an object class");
			throw new SnmpApplicationError("Trap could not be mapped to an object class");
		}

		System.err.println("SnmpAdapter: processTrapData> Trap is " + trapTypeToString(snmpTrapType) +
			" (trap type " + snmpTrapType +
			" ), thrown by objects of objectclass " + objectClass);

		NameComponent [] name = null;

		if (isRootObject(objectClass)) {
			name = findRegisteredRootObject(objectClass, agent.getHostAddress());
			if (name == null) {
				System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to a registered (root) object.");
				throw new SnmpApplicationError(ExceptionMessages.uknown_trap_origin_error);
			}
		}
		else {

			boolean valueIsNeeded = determineIfValueIsNeeded(objectClass);

			Any value = null;

			if (valueIsNeeded == true) {
				value = determineValue(objectClass, dataSetType);

				if (value == null) {
					System.err.println("SnmpAdapter: processTrapData> Could not determine value for trap!");
					throw new SnmpApplicationError(ExceptionMessages.trap_misconfig_error);
				}
			}

			name = findRegisteredLeafObject(objectClass, agent.getHostAddress(), value);
			if (name == null) {
				System.err.println("SnmpAdapter: processTrapData> Trap could not be mapped to a registered (leaf) object.");
				throw new SnmpApplicationError(ExceptionMessages.uknown_trap_origin_error);
			}
		}

		String objectFacadeKind = Utils.name2facade(Utils.name2string(name));
		String facadeNameKind = null;
		boolean facadeFound = false;

		System.err.println("SnmpAdapter: processTrapData> Object with name " + Utils.name2string(name) +
			" is accessible through a facade with kind " + objectFacadeKind);

		int index = 0;
		String facadeName = new String();
		for (index = 0; index < trap_target_facades.length; index++) {

			facadeName = trap_target_facades[index].objref.facade_name();
			facadeNameKind = Utils.name2facade(facadeName);

			//System.err.println("Checking with facade with name " +
			//	facadeName + " which has a kind " + facadeNameKind);

			if (objectFacadeKind.equals(facadeNameKind)) {

				System.err.println("SnmpAdapter: processTrapData> Object with name " +
					  Utils.name2string(name) +
					" is accessible through facade with name " + facadeName +
					    " found at position " + index);

				facadeFound = true;

				break;

			}
		}

		if (facadeFound == false) {

			System.err.println("SnmpAdapter: processTrapData> Could not find a facade to send trap to");
			throw new SnmpApplicationError("No Facade trap sink found");
		}

		int varBindNumber = pdu.getLength();

		DataTypeImpl [] trapData = new DataTypeImpl[varBindNumber];

		for (int j = 0; j < varBindNumber; j++) {

			SnmpVarBind vb = pdu.getVarBindAt(j);

			trapData[j] = new DataTypeImpl(vb.getName().toString(),
				convertSnmpSyntaxToAny(vb.getValue()),
				vb.getValue().typeId());

		}

		String community_str = (community != null) ?
							new String(community.getString()) : new String();

		//System.err.println("Calling notifyTrap with arguments: ");
		//System.err.println("Name: " + name);
		//System.err.println("Community: " + community_str);
		//System.err.println("RequestId: " + pdu.getRequestId());
		//System.err.println("SnmpTrapType: " + snmpTrapType);
		//System.err.println("Length of trap data: " + trapData.length);

		try {
			ist.coach.coachEmfServices.SnmpAdapter.TrapCallback facade_callback =
						trap_target_facades[index].objref;

			if (facade_callback != null) {
				facade_callback.notifyTrap( name,
										community_str,
										pdu.getRequestId(),
										snmpTrapType,
										trapData);
			}
			else
				System.err.println("SnmpAdapter: processTrapData> Could not retrieve reference to facade of " +
					Utils.name2string(name));
		}
		catch(SnmpApplicationError snmp_ex) {
			System.err.println("SnmpAdapterProvider: SnmpApplicationError exception caught " +
				" while trying to forward trap to " + Utils.name2string(name));
		}
		catch(intt.itu.itut_x780.ApplicationError app_exc) {
			System.err.println("SnmpAdapterProvider: ApplicationError exception caught " +
				" while trying to forward trap to " + Utils.name2string(name));
		}

    }

    private int determineTrapType(String trapObjectIdentifier) {


		int snmpTrapType = UNKNOWN_TRAP_TYPE;

		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (trapObjectIdentifier.equals(
				this.config.availableTraps[i].oid.toString())) {

				snmpTrapType = this.config.availableTraps[i].identifier;
				break;
			}
		}

		return snmpTrapType;

    }

    private String determineObjectClass(int snmpTrapType) {

		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (snmpTrapType == this.config.availableTraps[i].identifier) {
				return this.config.availableTraps[i].objectclass;
			}
		}

		return null;

    }

    private boolean determineIfValueIsNeeded(String objectClass) {

		boolean valueNeeded = false;

		for (int i = 0; i < this.config.availableTraps.length; i++) {

			if (objectClass.equalsIgnoreCase(this.config.availableTraps[i].objectclass)) {

				if (this.config.availableTraps[i].valueOid == null) {
					System.err.println("SnmpAdapter: determineIfValueIsNeeded> No value needed to fetch " +
						"leaf object of objectclass " + objectClass);
					valueNeeded = false;
				}
				else
					valueNeeded = true;
			}
		}

		return valueNeeded;
    }


    private Any determineValue(String objectClass, DataType [] varBindings) {

		String valueOid = null;

		//System.err.println("SnmpAdapter: determineValue> Determining value for objectClass..." +
		//	objectClass + ", with " + varBindings.length + " variable bindings");


		for (int i = 0; i < this.config.availableTraps.length; i++) {

			if (objectClass.equalsIgnoreCase(this.config.availableTraps[i].objectclass)) {

				if (this.config.availableTraps[i].valueOid != null)
					valueOid = new String(this.config.availableTraps[i].valueOid.toString());
				//else
				//	System.err.println("SnmpAdapter: determineValue> No value need to fetch leaf object of objectclass "
				//	  + objectClass);

				break;
			}

		}

		if (valueOid != null) {
			for (int i = 0; i < varBindings.length; i++) {
				if (varBindings[i].identifier.equals(
					valueOid)) {
					return (varBindings[i].value);
				}
			}
		}
		//System.err.println("SnmpAdapter: determineValue> Returning null");
		return null;

    }

    private String trapTypeToString(int snmpTrapType) {
		for (int i = 0; i < this.config.availableTraps.length; i++) {
			if (this.config.availableTraps[i].identifier == snmpTrapType) {

				return this.config.availableTraps[i].id;
			}

		}

		return null;
    }

	/**
	 * @param objectClass
	 * @param ipAddress
	 * @return NameComponent[]
	 *
	 * Returns the name of a root-object with the given
	 * objectclass and IP Address.
	 */
    private NameComponent[] findRegisteredRootObject(
		String objectClass, String ipAddress) {

		ListIterator li = this.linkedList.listIterator();

		//System.err.println("SnmpAdapter: findRegisteredRootObject> Trying to find Root object with object class " +
		//	objectClass + " and IP Address " + ipAddress);

		while (li.hasNext()) {

			ObjectData objData = (ObjectData)li.next();

			NameComponent [] name = objData.getName();
			String myObjectClass = name[name.length - 2].kind;
			String myIpAddress = objData.getIPAddress();

			//System.out.println("Comparing to object class " +
				//myObjectClass + " and IP Address " + myIpAddress);

			if (myObjectClass.equals(objectClass) &&
				myIpAddress.equals(ipAddress)) {

				//System.err.println("-----> Matched with object named " +
				//	Utils.name2string(objData.getName()));
				return objData.getName();
			}
		}

		return null;
    }

	/**
	 * @param objectClass
	 * @param ipAddress
	 * @param value
	 * @return NameComponent[]
	 *
	 * Returns the name of a registered leaf object with
	 * the given name, IP Address, registered with the
	 * given value.
	 */

    private NameComponent[] findRegisteredLeafObject(
		String objectClass, String ipAddress, Any value) {

		ListIterator li = this.linkedList.listIterator();

		//System.err.println("SnmpAdapter> Trying to find Leaf object with object class " +
		//		objectClass + " and IP Address " + ipAddress);

		while (li.hasNext()) {

			ObjectData objData = (ObjectData)li.next();

			NameComponent [] name = objData.getName();
			String myObjectClass = getObjectClass(name);
			String myIpAddress = objData.getIPAddress();

			if (myObjectClass.equals(objectClass) &&
				myIpAddress.equals(ipAddress)) {

				if (value != null) {
					if (objData.value.equal(value)) {

						//System.err.println("-----> Matched with object named" +
						//	Utils.name2string(objData.getName()));
						return objData.getName();
					}
				}
				else {
					//System.err.println("-----> Matched with object named" +
					//	Utils.name2string(objData.getName()));
					return objData.getName();
				}
			}
		}

		return null;
    }

	/**
	 * @param objectClass
	 * @return boolean
	 *
	 * Returns whether an object with the given objectclass
	 * is considered to be a root object or not.
	 */

    private boolean isRootObject(String objectClass) {

		for (int i = 0; i < config.rootObjects.length; i++)
			if (config.rootObjects[i].equals(objectClass))
				return true;

		return false;
    }

    private String getObjectClass(NameComponent[] name) {
		return name[name.length - 2].kind;
    }

    private boolean nameExists(NameComponent[] name) {

		ListIterator li = this.linkedList.listIterator();

		while (li.hasNext()) {
			ObjectData objData = (ObjectData) li.next();
			if (objData.getName().equals(name)) {
				return true;
			}
		}
		return false;
   	 }
    private boolean rootObjectExists(String ipAddress,
		NameComponent[] name) {

		ListIterator li = this.linkedList.listIterator();

		while (li.hasNext()) {

			ObjectData objData = (ObjectData)li.next();
			//System.err.println("Examining object with IPAddress = *" +
			  //objData.getIPAddress() + "* against *" + ipAddress + "*");

			if (objData.getIPAddress().equals(ipAddress) &&
				isRootObject(getObjectClass(objData.getName())) &&
				Utils.isChild(name, objData.getName()))
					return true;
		}

		return false;
    }



    private ObjectData findObjectWithName(NameComponent[] name) {

		String name_str = Utils.name2string(name);

		ListIterator li = this.linkedList.listIterator();

		ObjectData objData = null;

		while (li.hasNext()) {
			objData = (ObjectData) li.next();
			if (Utils.name2string(objData.getName()).equals(name_str)) {
				return objData;
			}
		}
		return null;
   	 }

    private class ObjectData {

		private NameComponent[] name;
		private String IPAddress;
		private Any value;

		ObjectData(NameComponent[] name, String IPAddress, Any value) {
			this.name = name;
			this.IPAddress = IPAddress;
			this.value = value;
		}

		ObjectData (NameComponent[] name, String IPAddress) {
			this.name = name;
			this.IPAddress = IPAddress;
			this.value = null;
		}

		public NameComponent[] getName() {
			return this.name;
		}

		public String getIPAddress() {
			return this.IPAddress;
		}
   	 }

    // ==================================================================
    //
    // Runnable Interface methods.
    //
    // ===================================================================
    public void run() {

		Thread myThread = Thread.currentThread();
		SnmpTrapSession trapSession = null;
		while (listenerThread == myThread) {

			try {
				trapSession = new SnmpTrapSession(this,
							SNMP_TRAP_PORT);
				System.err.println("SNMP Trap Receiver Started");
				synchronized(trapSession)
				{
					trapSession.wait();
				}
			}
			catch (InterruptedException ie) {
				System.err.println("SNMP Trap Receiver Exiting");
				trapSession.close();
			}
			catch (Exception e) {
				System.err.println("Exception in init(): " + e);
				e.printStackTrace();
			}
		}
   	 }


    // ==================================================================
    //
    // SnmpTrapHandler Interface methods.
    //
    // ===================================================================


    /**
     * Method from SnmpHandler interface, used to parse SNMPv2 Traps
     */
    public void snmpReceivedTrap(
		 SnmpTrapSession 	session,
		 java.net.InetAddress 	agent,
		 int 			port,
		 SnmpOctetString	community,
		 SnmpPduPacket 		pdu) {

		System.err.println("V2 Trap from agent " + agent.toString() + " on port " + port);
		System.err.println("V2 Trap Community........... " + new String(community.getString()));
		System.err.println("V2 Trap PDU command......... " + pdu.getCommand());
		System.err.println("V2 Trap PDU ID.............. " + pdu.getRequestId());
		System.err.println("V2 Trap PDU Length.......... " + pdu.getLength());

		if(pdu instanceof SnmpPduRequest)
		{
			System.err.println("V2 Trap PDU Error Status.... " + ((SnmpPduRequest)pdu).getErrorStatus());
			System.err.println("V2 Trap PDU Error Index..... " + ((SnmpPduRequest)pdu).getErrorIndex());
		}

		int k = pdu.getLength();

		for (int i = 0; i < k ; i++ )
		{
			SnmpVarBind vb = pdu.getVarBindAt(i);
			System.err.print("Varbind[" + i + "] := " + vb.getName().toString());
			System.err.println(" --> " + vb.getValue().toString());
		}

		try {

			processTrapData(agent, community, pdu);
		}
		catch (SnmpApplicationError e) {
			System.err.println("Unable to process trap!");
		}

   	 }

	/**
	 * Method from SnmpHandler interface used to parse SNMPv1 traps.
	 *
	 * @param session	The Trap Session that received the PDU.
	 * @param agent		The address of the remote sender.
	 * @param port		The remote port where the pdu was transmitted from.
	 * @param community	The decoded community string.
	 * @param pdu		The decoded V1 trap pdu.
	 *
	 */
    public void snmpReceivedTrap(
		SnmpTrapSession 	session,
		 java.net.InetAddress 	agent,
		 int 			port,
		 SnmpOctetString 	community,
		 SnmpPduTrap 		pdu)

	{

		System.err.println("V1 Trap from agent " + agent.toString() + " on port " + port);
		System.err.println("Ip Address................. " + pdu.getAgentAddress() );
		System.err.println("Enterprise Id.............. " + pdu.getEnterprise() );
		System.err.println("Generic ................... " + pdu.getGeneric() );
		System.err.println("Specific .................. " + pdu.getSpecific() );
		System.err.println("TimeStamp ................. " + pdu.getTimeStamp() );
		System.err.println("Length..................... " + pdu.getLength() );


		int k = pdu.getLength();
		for (int i = 0; i < k ; i++ )
		{
			SnmpVarBind vb = pdu.getVarBindAt(i);
			System.err.print("Varbind[" + i + "] := " + vb.getName().toString());
			System.err.println(" --> " + vb.getValue().toString());
		}
		System.err.println("");

		//synchronized(session)
		//{
		//session.notify();
		//}
   	 }

	/**
	 * Process session errors.
	 *
	 * @param session	The trap session in error.
	 * @param error		The error condition.
	 * @param ref		The reference object, if any.
	 */
    public void snmpTrapSessionError(
		SnmpTrapSession 	session,
		int 			error,
 		java.lang.Object 	ref)
	{

		System.err.println("SnmpAdapter:snmpTrapSessionError> An error occured in the trap session");
		System.err.println("Session error code = " + error);
		if(ref != null)
		{
			System.err.println("SnmpAdapter:snmpTrapSessionError> Session error reference: " + ref.toString());
		}

		if(error == SnmpTrapSession.ERROR_EXCEPTION)
		{
			synchronized(session)
			{
				session.notify(); // close the session
			}
		}
   	 }

    // ==================================================================
    //
    // Public methods.
    //
    // ===================================================================
    /**
     **  To obtain the context.
     **
     **  @return The context that has been previously set by
     **          the set_session_context operation.
     **/
    public CCM_SnmpAdapterProvider_Context
    getContext()
    {
        return the_context_;
    }
    // ==================================================================
    //
    // Methods for the OMG IDL Components::EnterpriseComponent local interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/Components/EnterpriseComponent/configuration_complete:1.0
    //
    /**
     **  Completes the component configuration.
     **
     **  @throws org.omg.Components.InvalidConfiguration
     **          Thrown if the configuration is invalid.
     **/
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
        //
        //  DONE : implement !!!
        //

		this.config = Configuration.getInstance();

		if (config == null) {
			System.err.println("Could not create a valid configuration. Exiting...");
			throw new org.omg.Components.InvalidConfiguration();
		}

		this.orb = org.objectweb.ccm.CORBA.TheORB.getORB();
		listener_init();


		if ((trap_target_facades =  the_context_.get_connections_trap_target()) == null) {
			System.err.println("SnmpAdapter is not configured with Facades to serve....");
		}

		System.err.println("SnmpAdapterProvider configuration completed...");
   	 }
    // ==================================================================
    //
    // Methods for the OMG IDL Components::SessionComponent local interface.
    //
    // ==================================================================
    //
    //  IDL:omg.org/Components/SessionComponent/set_session_context:1.0
    //
    /**
     *  Called by the container when the component session context will be fixed.
     *
     *  @param context The session context.
     *
     *  @throws org.omg.Components.CCMException
     *          Thrown if a system level error occured.
     */
    public void
    set_session_context(org.omg.Components.SessionContext context)
    throws org.omg.Components.CCMException
    {
        the_context_ = (CCM_SnmpAdapterProvider_Context)context;
    }
    //
    //  IDL:omg.org/Components/SessionComponent/ccm_activate:1.0
    //
    /**
     *
     * Called by the container when the component will be activated.
     *
     *  @throws org.omg.Components.CCMException
     *          Thrown if a system level error occured.
     */
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        //
        // Unused by actual OpenCCM containers
        //
    }
    //
    //  IDL:omg.org/Components/SessionComponent/ccm_passivate:1.0
    //
    /**
     *
     * Called by the container when the component will be passivated.
     *
     *  @throws org.omg.Components.CCMException
     *          Thrown if a system level error occured.
     */
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        //
        // Unused by actual OpenCCM containers
        //
    }
    //
    //  IDL:omg.org/Components/SessionComponent/ccm_remove:1.0
    //
    /**
     *
     * Called by the container when the component will be removed.
     *
     *  @throws org.omg.Components.CCMException
     *          Thrown if a system level error occured.
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        //
        // DONE: implement !!!
        //

		listener_stop();
    }
    // ==================================================================
    //
    // Public methods for the CCM_SnmpAdapterProvider local interface.
    //
    // ==================================================================
    //
    //  IDL:coach.ist/coachEmfServicesComponents/CCM_SnmpAdapterProvider/get_connector:1.0
    //
    /**
     **  Implementation of the ::coachEmfServicesComponents::CCM_SnmpAdapterProvider::get_connector operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.CCM_SnmpConnector
    get_connector()
    {
        return this;
    }
    //
    //  IDL:coach.ist/coachEmfServicesComponents/CCM_SnmpAdapterProvider/get_config:1.0
    //
    /**
     **  Implementation of the ::coachEmfServicesComponents::CCM_SnmpAdapterProvider::get_config operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.CCM_SnmpTrapDaemon
    get_config()
    {
        return this;
    }
    // ==================================================================
    //
    // Public methods for the ::coachEmfServices::SnmpAdapter::CCM_SnmpConnector local interface.
    //
    // ==================================================================
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/SnmpConnector/getSnmpTable:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::SnmpConnector::getSnmpTable operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.DataType[]
    getSnmpTable(java.lang.String tableStart,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //


	SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, tableStart);

	snmpConverter.snmpWalk();

	if (snmpConverter.agentResponded() == false) {

		System.err.println("Agent did not respond!");
		throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
	}


	DataTypeImpl [] snmpTable = new DataTypeImpl[snmpConverter.resultSet.size()];

	for (int i = 0; i < snmpConverter.resultSet.size(); i++) {

		SnmpVarBind vb = (SnmpVarBind)snmpConverter.resultSet.get(i);
		Any value = this.orb.create_any();
		value = convertSnmpSyntaxToAny(vb.getValue());
		if (value == null) {
			System.err.println("SnmpAdapter>Could not encapsulate SNMP attribute value into a CORBA Any");
			throw new SnmpApplicationError(ExceptionMessages.agent_create_error);
		}
		else  {
			snmpTable[i] = new DataTypeImpl(vb.getName().toString(), value, vb.getValue().typeId());

		}
	}

		return snmpTable;
    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/SnmpConnector/genNext:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::SnmpConnector::genNext operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.DataType
    getNext(java.lang.String identifier,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //

	SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, null);

	snmpConverter.snmpGetNext(new SnmpObjectId(identifier));

	if (snmpConverter.agentResponded() == false) {

		System.err.println("Agent did not respond!");
		throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
	}

	if (snmpConverter.resultSet.size() == 0) {

		System.err.println("Could not retrieve attribute with oid " + identifier);
		throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
	}

	Any value = this.orb.create_any();
	SnmpVarBind vb = (SnmpVarBind)snmpConverter.resultSet.elementAt(0);
	String oid = vb.getName().toString();
	value = convertSnmpSyntaxToAny(vb.getValue());

	if (value == null) {
		System.err.println("Could not retrieve attribute with oid " + identifier);
		throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
	}

	DataTypeImpl result = new DataTypeImpl(oid, value, vb.getValue().typeId());

	return result;

    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/Connector/get:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::Connector::get operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.DataType
    get(java.lang.String identifier,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //

	SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, null);

	SnmpVarBind [] vblist = new SnmpVarBind[1];
	vblist[0] = new SnmpVarBind(new SnmpObjectId(identifier));

	snmpConverter.snmpGet(vblist);

	if (snmpConverter.agentResponded() == false) {

		System.err.println("Agent did not respond!");
		throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
	}

	if (snmpConverter.resultSet.size() == 0) {

		System.err.println("Could not retrieve attribute with oid " + identifier);
		throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
	}

	//System.err.println("Got " + ((SnmpVarBind)snmpConverter.resultSet.elementAt(0)).toString());

	Any value = this.orb.create_any();
	SnmpVarBind vb = (SnmpVarBind)snmpConverter.resultSet.elementAt(0);
	String oid = vb.getName().toString();
	value = convertSnmpSyntaxToAny(vb.getValue());

	if (value == null) {
		System.err.println("Could not retrieve attribute with oid " + identifier);
		throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
	}

	DataTypeImpl result = new DataTypeImpl(oid, value, vb.getValue().typeId());

	return result;
    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/Connector/getBulk:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::Connector::getBulk operation.
     **/
    public ist.coach.coachEmfServices.SnmpAdapter.DataType[]
    getBulk(java.lang.String[] identifiers,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //
	SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, null);

	int numRequested = identifiers.length;

	SnmpVarBind [] vblist = new SnmpVarBind[numRequested];

	for (int i = 0; i < numRequested; i++)
		vblist[i] = new SnmpVarBind(new SnmpObjectId(identifiers[i]));

	snmpConverter.snmpGet(vblist);

	if (snmpConverter.agentResponded() == false) {

		System.err.println("Agent did not respond!");
		throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
	}

	if (snmpConverter.resultSet.size() == 0) {
		System.err.println("Could not retrieve attributes with requested identifiers");
		throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
	}

	DataTypeImpl [] result = new DataTypeImpl[snmpConverter.resultSet.size()];


	for (int i = 0; i < snmpConverter.resultSet.size(); i++) {

		Any value = this.orb.create_any();
		SnmpVarBind vb = (SnmpVarBind)snmpConverter.resultSet.elementAt(i);
		String oid = vb.getName().toString();
		value = convertSnmpSyntaxToAny(vb.getValue());

		if (value == null) {
			System.err.println("Could not encapsulate SNMP attribute value into a CORBA Any");
			throw new SnmpApplicationError(ExceptionMessages.agent_create_error);
		}
		else
			result[i] = new DataTypeImpl(((SnmpVarBind)snmpConverter.resultSet.elementAt(i)).getName().toString(), value, vb.getValue().typeId());
	}

	//System.err.println("getBulk is ready to return!!");
	return result;

    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/Connector/set:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::Connector::set operation.
     **/
    public void
    set(ist.coach.coachEmfServices.SnmpAdapter.DataType data,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //
		SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, null);

		SnmpVarBind [] vblist = new SnmpVarBind[1];
		SnmpSyntax value = convertAnyToSnmpSyntax(data.value, data.code);
		if (value == null) {
			System.err.println("Could not convert DataType to an SnmpSyntax");
			throw new SnmpApplicationError(ExceptionMessages.agent_create_error);
		}

		vblist[0] = new SnmpVarBind(new SnmpObjectId(data.identifier), value);

		snmpConverter.snmpSet(vblist);

		if (snmpConverter.agentResponded() == false) {

			System.err.println("Agent did not respond!");
			throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
		}

		if (snmpConverter.resultSet.size() == 0) {

			System.err.println("Could not set attribute with oid " + data.identifier);
			throw new SnmpApplicationError(ExceptionMessages.agent_set_error);
		}

	}
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/Connector/setBulk:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::Connector::setBulk operation.
     **/
    public void
    setBulk(ist.coach.coachEmfServices.SnmpAdapter.DataType[] data,
				java.lang.String host,
				int port,
				java.lang.String community)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //
		SnmpProtocolConverter snmpConverter = new SnmpProtocolConverter(host, community, port, null);
		int numRequested = data.length;
		//System.err.println("SnmpAdapter:setBulk> Data size = " + numRequested);
		SnmpVarBind [] vblist = new SnmpVarBind[numRequested];

		for (int i = 0; i < numRequested; i++) {
		    //System.err.println("[" + i + "] = ");

			SnmpSyntax value = convertAnyToSnmpSyntax(data[i].value, data[i].code);
			vblist[i] = new SnmpVarBind(new SnmpObjectId(data[i].identifier), value);
			//System.err.println("identifier = " +
			//	  data[i].identifier + " value = " + value);
		}

		snmpConverter.snmpSet(vblist);

		if (snmpConverter.agentResponded() == false) {

			System.err.println("Agent did not respond!");
			throw new SnmpApplicationError(ExceptionMessages.agent_response_error);
		}

		if (snmpConverter.resultSet.size() == 0) {
			System.err.println("setBulk>Could not set attributes");

			throw new SnmpApplicationError(ExceptionMessages.agent_get_error);
		}
	  //System.err.println("setBulk is ready to return");
	}
    // ==================================================================
    //
    // Public methods for the ::coachEmfServices::SnmpAdapter::CCM_SnmpTrapDaemon local interface.
    //
    // ==================================================================
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/SnmpTrapDaemon/registerIP:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::SnmpTrapDaemon::registerIP operation.
     **/
    public void
    registerIP(org.omg.CosNaming.NameComponent[] name, java.lang.String ipAddress)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //
	if (nameExists (name)) {
		System.err.println("Could not register object with Name " +
				Utils.name2string(name) + " because it already exists!");
		throw new SnmpApplicationError(ExceptionMessages.snmp_register_error);

	}
	else {
		ObjectData objData = new ObjectData (name, ipAddress);
		this.linkedList.add(objData);
		//System.err.println("SnmpAdapter> Registered object with Name " +
			//Utils.name2string(name) + " and IP Address *" + ipAddress + "*");
	}
    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/SnmpTrapDaemon/registerValue:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::SnmpTrapDaemon::registerValue operation.
     **/
    public void
    registerValue(org.omg.CosNaming.NameComponent[] name, java.lang.String ipAddress, org.omg.CORBA.Any value)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //
	//System.err.println("SnmpAdapter:registerValue> is called for " +
	//  Utils.name2string(name) + " with IP Address *" + ipAddress + "*");
	if (nameExists (name)) {
		System.err.println("SNMPAdapter:registerValue> Could not register object with Name "
		  + Utils.name2string(name) + " because it already exists!");
		throw new SnmpApplicationError(ExceptionMessages.snmp_register_error);
	}
	else
		if (rootObjectExists(ipAddress, name) == false) {
			System.err.println("SNMPAdapter:registerValue> Could not find parent for this object.");
			throw new SnmpApplicationError(ExceptionMessages.snmp_superior_error);
		}
		else {
			ObjectData objData = new ObjectData (name, ipAddress, value);
			this.linkedList.add(objData);
			//System.err.println("Registered object with Name " + Utils.name2string(name) + " and IP Address " + ipAddress);
		}
    }
    //
    //  IDL:coach.ist/coachEmfServices/SnmpAdapter/SnmpTrapDaemon/unregister:1.0
    //
    /**
     **  Implementation of the ::coachEmfServices::SnmpAdapter::SnmpTrapDaemon::unregister operation.
     **/
    public void
    unregister(org.omg.CosNaming.NameComponent[] name)
    throws ist.coach.coachEmfServices.SnmpAdapter.SnmpApplicationError
    {
        //
        //  DONE : implement
        //

	ObjectData objData = null;

	if ((objData = findObjectWithName(name)) != null) {
		this.linkedList.remove(objData);
		System.err.println("SnmpAdapter:unregister>Unregistered object with name " + Utils.name2string(name));
	}
	else {
		System.err.println("SnmpAdapter:unregister>Could not unregister object with name " + Utils.name2string(name));
		throw new SnmpApplicationError(ExceptionMessages.snmp_unregister_error);
	}

    }
}
